within ;
package DYNARLib
   class dynar
    extends ExternalObject;
    function constructor "Construct new dynamic array"
      input Integer x = 1;
      input Integer y = 1;
      input Integer z = 1;
      output dynar d;
      external "C" d = dynarCreate(x,y,z);
      annotation (
        Include = "#include \"dynarlib.c\"");
    end constructor;

    function destructor "Remove dynamnic array"
      input dynar d;

      external "C" dynarDelete(d);
      annotation (
        Include = "#include \"dynarlib.c\"");
    end destructor;
   end dynar;

   function dynarS "dynamic array into String"
     input dynar d;
     output String out;
     external "C" out = dynarString(d);
     annotation(Include = "#include \"dynarlib.c\"");
   end dynarS;

  function dynarSalt
      input dynar d;
     output String out;
  protected
     Integer x,y,z;
  algorithm
     x :=dynarX(d);
     y :=dynarY(d);
     z :=dynarZ(d);
     out := String(x) + " " + String(y) + " " + String(z);
     for i in 1:x loop
       for j in 1:y loop
         for k in 1:z loop
           out :=out + " " + String(dynarReadPos(
                d,
                i,
                j,
                k));
         end for;
       end for;
     end for;
  end dynarSalt;

   function Sdynar "String into dynamic array"
     input String s;
     input Integer indexi = 1;
     output dynar d;
     output Integer indexo;
  protected
     Integer x,y,z;
     Real v;
   algorithm
     indexo := indexi;
     (x,indexo):= Modelica.Utilities.Strings.scanInteger(s,indexo);
     (y,indexo):= Modelica.Utilities.Strings.scanInteger(s,indexo);
     (z,indexo):= Modelica.Utilities.Strings.scanInteger(s,indexo);
      dynarX(d,x);
      dynarY(d,y);
      dynarZ(d,z);
      for i in 1:x loop
        for j in 1:y loop
          for k in 1:z loop
            (v,indexo):= Modelica.Utilities.Strings.scanReal(s,indexo);
            dynarSetPos(
                 d,
                 v,
                 i,
                 j,
                 k);
          end for;
        end for;
      end for;
   end Sdynar;

     function dynarSetPos "set new value for a position"
      input dynar d;
       input Real newvalue;
       input Integer xpos = 1;
       input Integer ypos = 1;
       input Integer zpos = 1;
     external "C" dynarSetPos(d,newvalue,xpos,ypos,zpos);
      annotation (
        Include = "#include \"dynarlib.c\"");
     end dynarSetPos;

     function dynarSet "set new values for all positions"
      input dynar d;
       input Real newvalue[:,:,:];
       input Integer x = 1;
       input Integer y = 1;
       input Integer z = 1;
     external "C" dynarSet(d,newvalue,x,y,z);
      annotation (
        Include = "#include \"dynarlib.c\"");
     end dynarSet;

             function dynarRead
               input dynar d;
               output Real value[dynarX(d),dynarY(d),dynarZ(d)];
  protected
               constant Integer x = dynarX(d);
               constant Integer y = dynarY(d);
               constant Integer z = dynarZ(d);
             algorithm
               for i in 1:x loop
                 for j in 1:y loop
                   for k in 1:z loop
                     value[i,j,k] := dynarReadPos(d,i,j,k);
                   end for;
                 end for;
               end for;
             end dynarRead;

        function dynarReadPos
          input dynar d;
          input Integer xpos = 1;
          input Integer ypos = 1;
          input Integer zpos = 1;
          output Real value;
          external"C" value = dynarReadPos(
              d,
              xpos,
              ypos,
              zpos);
          annotation (
        Include = "#include \"dynarlib.c\"");
        end dynarReadPos;

  function dynarX
      input dynar d;
    input Integer newx = 0;
    output Integer x;
    external"C" x = dynarX(
              d,
              newx);
    annotation (
        Include = "#include \"dynarlib.c\"");
  end dynarX;

          function dynarY
    input dynar d;
            input Integer newy=0;
            output Integer y;
            external"C" y = dynarY(
              d,
              newy);
            annotation (Include="#include \"dynarlib.c\"");
          end dynarY;

          function dynarZ
    input dynar d;
            input Integer newz=0;
            output Integer z;
            external"C" z = dynarZ(
              d,
              newz);
            annotation (Include="#include \"dynarlib.c\"");
          end dynarZ;

   class dynarL "list of dynamic arrays"
    extends ExternalObject;
    function constructor "Construct new dynamic array list"
      output dynarL d;
      external "C" d = dynarLCreate();
      annotation (
        Include = "#include \"dynarlib.c\"");
    end constructor;

    function destructor "Remove dynamnic array"
      input dynarL d;

      external "C" dynarLDelete(d);
      annotation (
        Include = "#include \"dynarlib.c\"");
    end destructor;
   end dynarL;

   function dynarLS
     input dynarL l;
     output String s;
       external"C" s = dynarLString(l);
     annotation (Include="#include \"dynarlib.c\"");
   end dynarLS;

      function dynarLSalt
        input dynarL l;
        output String out;
  protected
        Integer size;
        dynar e;
        Integer number;
      algorithm
        size := dynarLSize(l);
        out := String(size);
        for i in 1:size loop
          (e,number) := dynarLReadPos(l,i);
          out :=out + " " + String(number) + " " + dynarSalt(e);
        end for;
      end dynarLSalt;

   function SdynarL
     input String s;
     input Integer indexi = 1;
     output dynarL l;
     output Integer indexo;
  protected
     Integer size;
     dynar e;
     Integer number;
   algorithm
     l := dynarL();
     indexo := indexi;
     (size,indexo) := Modelica.Utilities.Strings.scanInteger(s,indexo);
     for i in 1:size loop
       (number,indexo) := Modelica.Utilities.Strings.scanInteger(s,indexo);
       (e,indexo) := Sdynar(s,indexo);
       dynarLAddLast(l,number,e);
     end for;
   end SdynarL;

  function dynarLSize
      input dynarL l;
    output Integer size;
    external"C" size = dynarLSize(l);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLSize;

  function dynarLNum
      input dynarL l;
      input Integer pos;
    output Integer num;
    external"C" num = dynarLNum(l,pos);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLNum;

  function dynarLPos
      input dynarL l;
      input Integer num;
    output Integer pos;
    external"C" pos = dynarLPos(l,num);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLPos;

  function dynarLCopy
      input dynarL l;
      output dynarL copy;
  protected
    Integer size;
    Integer number;
    dynar d;
  algorithm
    copy := dynarL();
    size := dynarLSize(l);
    for i in 1:size loop
      number :=dynarLNum(l, i);
      d := dynarLReadPos(l, i);
      dynarLAddLast(copy,number,d);
    end for;
  end dynarLCopy;

  function dynarLCopyalt
      input dynarL l;
      output dynarL copy;
    external"C" dynarLCopy(l,copy);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLCopyalt;

  function dynarLAddFirst
    input dynarL l;
    input Integer number;
    input dynar e;
  external"C" dynarLAddFirst(l,number,e);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLAddFirst;

  function dynarLAddLast
    input dynarL l;
    input Integer number;
    input dynar e;
  external"C" dynarLAddLast(l,number,e);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLAddLast;

  function dynarLAddLNF
     input dynarL l;
    input Integer number;
    input dynar e;
  external"C" dynarLAddLNF(l,number,e);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLAddLNF;

  function dynarLAddHNF
    input dynarL l;
    input Integer number;
    input dynar e;
  external"C" dynarLAddHNF(l,number,e);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLAddHNF;

  function dynarLRemoveNum
      input dynarL d;
    input Integer number;
  external"C" dynarLRemoveNum(d,number);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLRemoveNum;

  function dynarLRemovePos
      input dynarL d;
    input Integer pos;
  external"C" dynarLRemovePos(d,pos);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLRemovePos;

  function dynarLReadNum
      input dynarL d;
    input Integer number;
      output dynar element;
  external"C" dynarLReadNum(d,number,element);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLReadNum;

  function dynarLReadNumXYZ
      input dynarL d;
    input Integer number;
    input Integer xpos = 1;
    input Integer ypos = 1;
    input Integer zpos = 1;
    output Real value;
  external"C" value = dynarLReadNumXYZ(d,number,xpos,ypos,zpos);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLReadNumXYZ;

  function dynarLReadPos
      input dynarL d;
      input Integer pos;
      output dynar element;
      output Integer number;
      external"C" number = dynarLReadPos(d,pos,element);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLReadPos;

  function dynarLReadPosXYZ
      input dynarL d;
    input Integer pos;
    input Integer xpos = 1;
    input Integer ypos = 1;
    input Integer zpos = 1;
    output Real value;
  external"C" value = dynarLReadPosXYZ(d,pos,xpos,ypos,zpos);
    annotation (Include="#include \"dynarlib.c\"");
  end dynarLReadPosXYZ;

  model test1
    Real [2,2,2] x;
    dynar d= dynar(2,2,2);
    Integer n;

  algorithm
    when time >= 1 then
      n := 1;
      for i in 1:2 loop
        for j in 1:2 loop
          for k in 1:2 loop
            x[i,j,k] := n;
            n := n+1;
          end for;
        end for;
      end for;
      dynarSet(d,x,2,2,2);
        Modelica.Utilities.Streams.print(dynarS(d));
    end when;
     annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(
          coordinateSystem(preserveAspectRatio=false)));
  end test1;

  model test2
    Real [2,2,2] x;
    dynar d= dynar(2,2,2);
    dynar dd;
    String s;
    Integer n;

  algorithm
    when time >= 1 then
      n := 1;
      for i in 1:2 loop
        for j in 1:2 loop
          for k in 1:2 loop
            x[i,j,k] := n;
            n := n+1;
          end for;
        end for;
      end for;
      dynarSet(d,x,2,2,2);
        Modelica.Utilities.Streams.print(dynarS(d));
    end when;
    when time >= 2 then
      for i in 1:2 loop
        for j in 1:2 loop
          for k in 1:2 loop
            dynarSetPos(d,dynarReadPos(d,i,j,k)*2,i,j,k);
          end for;
        end for;
      end for;
      s :=dynarS(d);
      Modelica.Utilities.Streams.print(s);
      dd :=Sdynar(s);
        Modelica.Utilities.Streams.print(dynarS(dd));
    end when;

     annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(
          coordinateSystem(preserveAspectRatio=false)),
      experiment(StopTime=3));
  end test2;

  model test3
    Real [2,2,2] x;
    dynar d= dynar(2,2,2);
    Integer n;

  algorithm
    when time >= 1 then
      n := 1;
      for i in 1:2 loop
        for j in 1:2 loop
          for k in 1:2 loop
            x[i,j,k] := n;
            n := n+1;
          end for;
        end for;
      end for;
      dynarSet(d,x,2,2,2);
        Modelica.Utilities.Streams.print(dynarS(d));
      x := zeros(2,2,2);
    end when;
    when time >= 2 then
      x := dynarRead(d);
    end when;

     annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(
          coordinateSystem(preserveAspectRatio=false)),
      experiment(StopTime=3));
  end test3;

  model testL1
    dynar d1= dynar(1,1,1);
    dynar d2= dynar(1,1,1);
    dynar d3= dynar(1,1,1);
    dynarL l = dynarL();
    dynar d;
    Real a;
  algorithm
    when time >= 1 then
      dynarSetPos(d1,1);
      dynarSetPos(d2,2);
      dynarSetPos(d3,3);
      Modelica.Utilities.Streams.print(dynarS(d1));
      Modelica.Utilities.Streams.print(dynarS(d2));
      Modelica.Utilities.Streams.print(dynarS(d3));
      dynarLAddHNF(l,3,d1);
      dynarLAddHNF(l,1,d2);
      dynarLAddHNF(l,2,d3);
      Modelica.Utilities.Streams.print(dynarLS(l));
    end when;
    when time >= 2 then
      dynarLRemoveNum(l,3);
      Modelica.Utilities.Streams.print(dynarLS(l));
      dynarLRemovePos(l,2);
      Modelica.Utilities.Streams.print(dynarLS(l));
      d := dynarLReadNum(l, 2);
      Modelica.Utilities.Streams.print(dynarS(d));
      a := dynarLReadNumXYZ(l,2,1,1,1);
      Modelica.Utilities.Streams.print(String(a));
      dynarLAddHNF(l,1,d2);
      Modelica.Utilities.Streams.print(dynarLS(l));
      d :=dynarLReadPos(l, 2);
      Modelica.Utilities.Streams.print(dynarS(d));
      a := dynarLReadPosXYZ(l,2,1,1,1);
      Modelica.Utilities.Streams.print(String(a));
    end when;

     annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(
          coordinateSystem(preserveAspectRatio=false)),
      experiment(StopTime=3));
  end testL1;

  model testL2
    dynar d1= dynar(1,1,1);
    dynar d2= dynar(1,1,1);
    dynar d3= dynar(1,1,1);
    dynarL l = dynarL();
    dynarL lc;
    dynar d;
    Real a;
  initial algorithm
    lc := dynarL();
  algorithm
    when time >= 1 then
      dynarSetPos(d1,1);
      dynarSetPos(d2,2);
      dynarSetPos(d3,3);
      Modelica.Utilities.Streams.print(dynarS(d1));
      Modelica.Utilities.Streams.print(dynarS(d2));
      Modelica.Utilities.Streams.print(dynarS(d3));
      dynarLAddHNF(l,3,d1);
      dynarLAddHNF(l,1,d2);
      dynarLAddHNF(l,2,d3);
      Modelica.Utilities.Streams.print(dynarLS(l));
    end when;
    when time >= 2 then
      dynarLRemoveNum(l,3);
      Modelica.Utilities.Streams.print(dynarLS(l));
      dynarLRemovePos(l,2);
      Modelica.Utilities.Streams.print(dynarLS(l));
      d := dynarLReadNum(l, 2);
      Modelica.Utilities.Streams.print(dynarS(d));
      a := dynarLReadNumXYZ(l,2,1,1,1);
      Modelica.Utilities.Streams.print(String(a));
      dynarLAddHNF(l,1,d2);
      Modelica.Utilities.Streams.print(dynarLS(l));
      d :=dynarLReadPos(l, 2);
      Modelica.Utilities.Streams.print(dynarS(d));
      a := dynarLReadPosXYZ(l,2,1,1,1);
      Modelica.Utilities.Streams.print(String(a));

      Modelica.Utilities.Streams.print("------------------------------");

      Modelica.Utilities.Streams.print(dynarLS(l));
      Modelica.Utilities.Streams.print(dynarLS(lc));
      lc := dynarLCopy(l);
      Modelica.Utilities.Streams.print(dynarLS(l));
      Modelica.Utilities.Streams.print(dynarLS(lc));

    end when;

     annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(
          coordinateSystem(preserveAspectRatio=false)),
      experiment(StopTime=3));
  end testL2;
  annotation (uses(Modelica(version="4.0.0")));
end DYNARLib;
