#ifndef DYNARLIB
#define DYNARLIB

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>


typedef struct{
    int x, y, z;
    double ***value;    
} dynar;

// constructor
void * dynarCreate(int x, int y, int z){
    int i,j;
    dynar *d = (dynar*)malloc(sizeof(dynar));
    d->value = malloc(sizeof(*d->value)*x);
    for (i = 0; i < x; i++){
	d->value[i] = malloc(sizeof(*(d->value[i]))*y);
	for (j = 0; j < y; j++){
	    d->value[i][j] = malloc(sizeof(*(d->value[i][j])) * z);
	}
    }    
    d->x = x;
    d->y = y;
    d->z = z;
    return (void *)d;
}


// destructor
void dynarDelete(void *o){
    dynar *d =(dynar *)o;
    int i;
    
    if (d != NULL){
	for (i = 0; i < d->x; i++){
	    //for (j = 0; j < d->y; j++)
	    //	free(d->value[i,j]);
	    free(d->value[i]);
	}
	free(d->value);
	free(d);
    }
    return;
}


char * dynarString(void *o){
    dynar *d;
    char *out;
    char number[31];
    int i,j,k;
    
    d = (dynar *)o;
    out = malloc(sizeof(char)*(21+(d->x*d->y*d->z)*60));
    sprintf(out,"%d %d %d ",d->x,d->y,d->z);
    for(i=0;i<d->x;i++)
	for(j=0;j<d->y;j++)
	    for(k=0;k<d->z;k++){
		sprintf(number,"%f ",d->value[i][j][k]);
		strcat(out,number);
	    }
    return out;
}



int dynarSetPos(void *o, double newvalue, int mposx, int mposy, int mposz){
    dynar *d =(dynar*)o;
    // modelica positions start from 1
    int posx = mposx-1;
    int posy = mposy-1;
    int posz = mposz-1;
    
        //printf("dynarsetpos %lf, pos [%d,%d,%d]\n",newvalue,posx,posy,posz);
    if (d != NULL){
	if (mposx > d->x || mposy > d->y || mposz > d->z)
	    return -2; //out of bounds
	else{
	    d->value[posx][posy][posz] = newvalue;
	    //ModelicaFormatMessage(dynarString(o));
	    //ModelicaFormatMessage("new = %lf\n",d->value[posx][posy][posz]);
	}return 1;
    }else
	return -1;
}

int dynarSet(void *o, double *newvalue, int x, int y, int z){
    dynar *d =(dynar*)o;
    // modelica positions start from 1
    int i,j,k;
    
    if (d != NULL){
	//	//printf("%d <= %d && %d <= %d && %d <= %d\n",x,d->x,y,d->y,z,d->z);
	if(x <= d->x && y<= d->y && z <= d->z){
	    //ModelicaFormatMessage("%d, %d, %d \n",d->x,d->y,d->z);
	    //for (i=0;i<x*y*z;i++)
	    //ModelicaFormatMessage("vector value[%d] = %lf\n",i,newvalue[i]);
	    //	    //printf("update with newvalue\n");
	    for(i=0;i<x;i++){
		for(j=0;j<y;j++){
		    for(k=0;k<z;k++){
			//ModelicaFormatMessage("newvalue[%d,%d,%d] = %lf\n",i,j,k,newvalue[i*(d->y*d->z)+j*(d->z)+k]);
			d->value[i][j][k] = newvalue[i*(d->y*d->z)+j*(d->z)+k];
			//ModelicaFormatMessage("d->value[%d,%d,%d] = %lf\n",i,j,k,d->value[i][j][k]);
		    }
		}
	    }
	    return 1;
	}else
	    return -2;
    }else
	return -1;
}

/*int dynarRead(void *o, double *value, int x, int y, int z){
    dynar *d =(dynar*)o;
    
    if (d != NULL){
	//	x = &(d->x);
	//y = &(d->y);
	//z = &(d->z);
	value = &(d->value[x][y][z]);
	return 1;
    }else
	return -1;
    
	}*/


double dynarReadPos(void *o, int mposx, int mposy, int mposz){
    dynar *d =(dynar*)o;
    // modelica positions start from 1
    int posx = mposx-1;
    int posy = mposy-1;
    int posz = mposz-1;

    if (d != NULL){
	////printf("dynarreadpos %d <= %d && %d <= %d && %d <= %d\n",mposx,d->x,mposy,d->y,mposz,d->z);
	if (mposx <= d->x && mposy <= d->y && mposz <= d->z){
	    //	    //printf("return value %p\n",d->value[0]);
	    //ModelicaFormatMessage("pos[%d,%d,%d] = %lf\n",mposx,mposy,mposz,d->value[posx][posy][posz]);
	    return d->value[posx][posy][posz];
	}else
	    return -1; // out of bounds
    }else
	return -1;
    
}

int dynarX(void *o, int newx){
    dynar *d =(dynar*)o;
    dynar *oldd = (dynar *)o;
    int i,j;
    // modelica positions start from 1

    //   //printf("dynarX----\n");
    if (d != NULL){
	//	//printf("newx = %d,  d->x = %d\n",newx,d->x);
	if (newx <= 0) // return current x
	    return d->x;
	else if (newx > d->x){ // increase array dimension
	    d->value = realloc(d->value,sizeof(*d->value)*newx);
	    for(i = 0; i < d->x ; i++)
		d->value[i] = oldd->value[i]; // store old values
	    for (i = d->x; i < newx; i++){ // memory for new dimensions
		d->value[i] = malloc(sizeof(*(d->value[i]))*d->y);
		for (j = 0; j < d->y; j++){
		    d->value[i][j] = malloc(sizeof(*(d->value[i][j])) * d->z);
		}
	    }
	    d->x = newx;
	    return 1;
	}
    }
    return -1;
}

int dynarY(void *o, int newy){
    dynar *d =(dynar*)o;
    dynar *oldd = (dynar *)o;
    int i,j;
    // modelica positions start from 1

    //   //printf("dynarX----\n");
    if (d != NULL){
	//	//printf("newx = %d,  d->x = %d\n",newx,d->x);
	if (newy <= 0) // return current x
	    return d->y;
	else if (newy > d->y){ // increase array dimension
	    for (i = 0; i < d->x; i++){ 
		d->value[i] = realloc(d->value[i],sizeof(*(d->value[i]))*newy);
		for (j = 0; j < d->y; j++){
		    d->value[i][j] = oldd->value[i][j];
		}
		for (j = d->y; j < newy; j++){
		    d->value[i][j] = malloc(sizeof(*(d->value[i][j])) * d->z);
		}
	    }
	    d->y = newy;
	    return 1;
	}
    }
    return -1;
}

int dynarZ(void *o, int newz){
    dynar *d =(dynar*)o;
    dynar *oldd = (dynar *)o;
    int i,j,k;
    // modelica positions start from 1

    //   //printf("dynarX----\n");
    if (d != NULL){
	//	//printf("newx = %d,  d->x = %d\n",newx,d->x);
	if (newz <= 0) // return current x
	    return d->z;
	else if (newz > d->z){ // increase array dimension
	    for (i = 0; i < d->x; i++){
		//printf("hola\n");
		for (j = 0; j < d->y; j++){
		    d->value[i][j] = realloc(d->value[i][j],sizeof(*(d->value[i][j])) * newz);		    
		    for(k = 0; k < d->z; k++){
			d->value[i][j][k] = oldd->value[i][j][k];
		    }
		}
	    }
	    d->z = newz;
	    return 1;
	}
    }
    return -1;
}





/*int main(){
    void *o;
    double v;
    //printf("createDynar\n");
    o = dynarCreate(2,2,2);

    double newv[2][2][2];
    
    newv[0][0][0] = 1;
    newv[0][0][1] = 2;
    newv[0][1][0] = 3;
    newv[0][1][1] = 4;
    newv[1][0][0] = 5;
    newv[1][0][1] = 6;
    newv[1][1][0] = 7;
    newv[1][1][1] = 8;

    //printf("setm\n");
    dynarSetM(o,&newv,2,2,2);
    //printf("after setm\n");
    //printf("dynar[1,1,1] = %lf\n",dynarRead(o,1,1,1));
    //printf("dynar[1,1,2] = %lf\n",dynarRead(o,1,1,2));
    //printf("dynar[1,2,1] = %lf\n",dynarRead(o,1,2,1));
    //printf("dynar[1,2,2] = %lf\n",dynarRead(o,1,2,2));
    //printf("dynar[2,1,1] = %lf\n",dynarRead(o,2,1,1));
    //printf("dynar[2,1,2] = %lf\n",dynarRead(o,2,1,2));
    //printf("dynar[2,2,1] = %lf\n",dynarRead(o,2,2,1));
    //printf("dynar[2,2,2] = %lf\n",dynarRead(o,2,2,2));
    
    //printf("-----\n");
    
    dynarSet(o,1,1,1,1);
    //printf("dynar[1,1,1] = %lf\n",dynarRead(o,1,1,1));
    //dynarSet(o,2,2,1,1);
    //printf("dynar[1,1,2] = %lf\n",dynarRead(o,1,1,2));

    //printf(" dim = %d\n",dynarY(o,0));
    dynarY(o,2);
    //printf(" new dim = %d\n",dynarY(o,0));
    //printf("dynar[1,1,1] = %lf\n",dynarRead(o,1,1,1));
    dynarSet(o,2,1,1,2);
    //printf("dynar[1,2,1] = %lf\n",dynarRead(o,1,2,1));

    newv[0][0][0] = 10;
    newv[0][0][1] = 20;
 
    //printf("setm\n");
    dynarSetM(o,&newv,1,1,2);
    //printf("after setm\n");
    //printf("dynar[1,1,1] = %lf\n",dynarRead(o,1,1,1));
    //printf("dynar[1,1,2] = %lf\n",dynarRead(o,1,1,2));
    //printf("dynar[1,2,1] = %lf\n",dynarRead(o,1,2,1));
    //printf("dynar[1,2,2] = %lf\n",dynarRead(o,1,2,2));
    //printf("dynar[2,1,1] = %lf\n",dynarRead(o,2,1,1));
    //printf("dynar[2,1,2] = %lf\n",dynarRead(o,2,1,2));
    //printf("dynar[2,2,1] = %lf\n",dynarRead(o,2,2,1));
    //printf("dynar[2,2,2] = %lf\n",dynarRead(o,2,2,2));
    
    //printf("-----\n");
     
    
    dynarDelete(o);
    
    return 0;
}
*/



/*******************************************************************************************/



typedef struct dynarLE{
    dynar e; //list element
    int number;
    struct dynarLE *prev; // previous element
    struct dynarLE *next; // next element
} dynarLE;

typedef struct{
    dynarLE *l;
    int size;
} dynarL;

// constructor
void * dynarLCreate(){
    dynarL *l;

    l = malloc(sizeof(dynarL));
    // empty list
    l->l = NULL; 
    l->size = 0;
    return (void *)l;
}


// destructor
void dynarLDelete(void *o){
    dynarL *l =(dynarL *)o;
    int i;
    dynarLE *e;

    if (l != NULL){
	for (i = 0; i < l->size; i++){
	    e  = l->l;
	    l->l = l->l->next;
	    free(e);
	}
	free(l);
    }
    return;
}

char * dynarLString(void *o){
    dynarL *l;
    dynarLE *pl;
    char *out;
    int size,i;
    char *dynar;
    char number[9];

 
    l = (dynarL *)o;
    //ModelicaFormatMessage("dynarLString size=%d \n",l->size);
    size = 9;
    pl = l->l;
    for(i=0;i<l->size;i++){
	size += (pl->e.x*pl->e.y*pl->e.z)*60;
	pl = pl->next;
    }
    out = malloc(sizeof(char)*size);
    sprintf(out,"%d ",l->size);
    pl = l->l;
    for(i=0;i<l->size;i++){
	sprintf(number,"%d ",pl->number);
	dynar = dynarString((void*)&(pl->e));
	strcat(out,number);
	strcat(out,dynar);
	pl = pl->next;
    }
    //ModelicaFormatMessage("dynarLString: -%s- \n",out);
    return out;
    
}

int dynarLSize(void *o){
    dynarL *l;

    l = (dynarL *)o;
    return l->size;
}


int dynarLNum(void *o,int pos){
    dynarL *l;
    dynarLE *pl;
    int i;
    
    l = (dynarL *)o;
    if (l != NULL){
	pl = l->l;
	i = 1;
	while (i<pos && pl != NULL){
	    i++;
	    pl = pl->next;
	}
	if (pl != NULL)
	    return pl->number;
    }
    return -1;
}


int dynarLPos(void *o,int num){
    dynarL *l;
    dynarLE *pl;
    int i;

    l = (dynarL *)o;
    if (l != NULL){
	pl = l->l;
	i = 1;
	while (pl->number!=num && pl != NULL){
	    i++;
	    pl = pl->next;
	}
	if (pl != NULL)
	    return i;
    }
    return -1;
}


int dynarLAddFirst(void *o, int number, void *e){
    dynarL *l;
    dynar *newe;
    dynarLE *newle;
    dynarLE *pl;

    l = (dynarL *)o; // list
    newe = (dynar *)e; // new dynar
    if (l != NULL){
	newle = malloc(sizeof(dynarLE));
	newle->number = number;
	newle->e = *(newe);
	pl = l->l;
	// insert first
	newle->next = pl;
	if (pl != NULL) pl->prev = newle;
	newle->prev = NULL;
	l->l = newle;
	l->size++;
	return 1;
    }
    return -1;
}

int dynarLAddLast(void *o, int number, void *e){
    dynarL *l;
    dynar *newe;
    dynarLE *newle;
    dynarLE *pl;

    l = (dynarL *)o;
    newe = (dynar *)e;
    if (l != NULL){
	newle = malloc(sizeof(dynarLE));
	newle->number = number;
	newle->e = *(newe);
	// find the end
	pl = l->l;
	newle->prev = pl;
	while(pl != NULL){
	    newle->prev = pl;
	    pl = pl->next;
	}
	// insert at the end
	newle->next = NULL;
	if (newle->prev != NULL)
	    newle->prev->next = newle;
	else
	    l->l = newle;
	//new->prev = pl;
	l->size++;
	return 1;
    }
    return -1;
}

// LOW NUMBER FIRST
int dynarLAddLNF(void *o, int number, void *e){
    dynarL *l;
    dynarLE *newle;
    dynarLE *pl;
    dynar *newe;
    
    l = (dynarL *)o;
    newe = (dynar *)e;
    if (l != NULL){
	newle = malloc(sizeof(dynarLE));
	newle->number = number;
	newle->e = *(newe);
	pl = l->l;
	if (pl == NULL || pl->number > number){ // empty list or insert first
	    newle->next = pl;
	    if (pl != NULL) pl->prev = newle;
	    newle->prev = NULL;
	    l->l = newle;
	    l->size++;
	}else{
	    newle->prev = pl;
	    pl = pl->next;
	    while(pl != NULL){
		if (pl->number == number){ // substitute element with new value
		    newle->prev = pl->prev;
		    newle->next = pl->next;
		    pl->prev->next = newle;
		    pl->next->prev = newle;
		    pl->prev = NULL;
		    pl->next = NULL;
		    free(pl);
		    return 1;
		}else if (pl->number > number){ // insert before pl
		    newle->next = pl;
		    newle->prev = pl->prev;
		    pl->prev->next = newle;
		    pl->prev = newle;
		    l->size++;
		    return 1;
		}else{
		    newle->prev = pl;
		    pl = pl->next;
		}
	    }
	    // insert at the end
	    newle->next = NULL;
	    newle->prev->next = newle;
	    //new->prev = pl;
	    l->size++;
	}
    }
    return 1;
}

// HIGH NUMBER FIRST
int dynarLAddHNF(void *o, int number, void *e){
    dynarL *l;
    dynarLE *newle;
    dynarLE *pl;
    dynar *newe;
    
    l = (dynarL *)o;
    newe = (dynar *)e;
    if (l != NULL){
	newle = malloc(sizeof(dynarLE));
	newle->number = number;
	newle->e = *(newe);
	pl = l->l;
	if (pl == NULL || pl->number < number){ // empty list or insert first
	    newle->next = pl;
	    if (pl != NULL) pl->prev = newle;
	    newle->prev = NULL;
	    l->l = newle;
	    l->size++;
	}else{
	    newle->prev = pl;
	    pl = pl->next;
	    while(pl != NULL){
		if (pl->number == number){ // substitute element with new value
		    newle->prev = pl->prev;
		    newle->next = pl->next;
		    pl->prev->next = newle;
		    pl->next->prev = newle;
		    pl->prev = NULL;
		    pl->next = NULL;
		    free(pl);
		    return 1;
		}else if (pl->number < number){ // insert before pl
		    newle->next = pl;
		    newle->prev = pl->prev;
		    pl->prev->next = newle;
		    pl->prev = newle;
		    l->size++;
		    return 1;
		}else{
		    newle->prev = pl;
		    pl = pl->next;
		}
	    }
	    // insert at the end
	    newle->next = NULL;
	    newle->prev->next = newle;
	    //new->prev = pl;
	    l->size++;
	}
    }
    return 1;
}


int dynarLRemoveNum(void *o,int number){
    dynarL *l;
    dynarLE *pl;
    
    l = (dynarL *)o;
    if (l != NULL){
	pl = l->l;
	while(pl != NULL){
	    if (pl->number == number){
		if (pl->prev != NULL)
		    pl->prev->next = pl->next;
		else
		    l->l = pl->next;
		if(pl->next != NULL)
		    pl->next->prev = pl->prev;
		free(pl);
		l->size--;
		return 1;
	    }else
		pl = pl->next;
	}
    }
    return -1;
}

int dynarLRemovePos(void *o,int pos){
    dynarL *l;
    dynarLE *pl;
    int i;
    
    l = (dynarL *)o;
    if (l != NULL){
	pl = l->l;
	//find pos
	i = 1;
	while(i<pos && i<l->size){
	    i++;
	    pl = pl->next;
	}
	if (i>l->size)
	    return -2; // pos out of size
	else{
	    if (pl->prev != NULL)
		pl->prev->next = pl->next;
	    else
		l->l = pl->next;
	    if(pl->next != NULL)
		pl->next->prev = pl->prev;
	    free(pl);
	    l->size--;
	    return 1;
	}
    }
    return -1;
}



int dynarLReadNum(void *o, int number, void *e){
    dynarL *l;
    dynarLE *pl;
    dynar *auxe;
    int i,j,k;

    
    l = (dynarL *)o;
    auxe = (dynar *)e;
    if (l != NULL){
	pl = l->l;
	while(pl != NULL){
	    if (pl->number == number){
		dynarX(e,pl->e.x);
		dynarY(e,pl->e.y);
		dynarZ(e,pl->e.z);
		for(i=0;i<pl->e.x;i++)
		    for(j=0;j<pl->e.y;j++)
			for(k=0;k<pl->e.z;k++)
			    auxe->value[i][j][k] = pl->e.value[i][j][k];
		
		return 1;
	    }else
		pl = pl->next;
	}
    }
    return -1; // error
}

double dynarLReadNumXYZ(void *o, int number, int mx, int my, int mz){
    dynarL *l;
    dynarLE *pl;
    
    l = (dynarL *)o;
    if (l != NULL){
	pl = l->l;
	while(pl != NULL){
	    if (pl->number == number)
		return dynarReadPos((void*)&(pl->e),mx,my,mz);
	    else
		pl = pl->next;
	}
    }
    return -1; // error
}

int dynarLReadPos(void *o, int pos, void *e){
    dynarL *l;
    dynarLE *pl;
    dynar *auxe;
    int i,j,k;
    
    l = (dynarL *)o;
    auxe = (dynar *)e;
    if (l != NULL){
	pl = l->l;
	//find pos
	i = 1;
	while(i<pos){// && i<l->size){
	    i++;
	    pl = pl->next;
	}
	if (i>l->size)
	    return -2; // pos out of size
	else{
	    dynarX(e,pl->e.x);
	    dynarY(e,pl->e.y);
	    dynarZ(e,pl->e.z);
	    for(i=0;i<pl->e.x;i++)
		for(j=0;j<pl->e.y;j++)
		    for(k=0;k<pl->e.z;k++)
			auxe->value[i][j][k] = pl->e.value[i][j][k];
	    return pl->number;
	}
    }
    return -1; // error
}

double dynarLReadPosXYZ(void *o, int pos, int mx, int my, int mz){
    dynarL *l;
    dynarLE *pl;
    int i;
    
    l = (dynarL *)o;
    if (l != NULL){
	pl = l->l;
	//find pos
	i = 1;
	while(i<pos){ // && i<l->size){
	    i++;
	    pl = pl->next;
	}
	if (i>l->size)
	    return -2; // pos out of size
	else
	    return dynarReadPos((void*)&(pl->e),mx,my,mz);
    }
    return -1; // error
}

int dynarLCopy(void*o,void*d){
    dynarL *lo, *ld;
    dynarLE *plo;
    void *auxe;
    int i,number;

    //dynarLDelete(d);
    d = dynarLCreate();
    lo = (dynarL *)o;
    ld = (dynarL *)d;
    if (ld != NULL && ld != NULL){
	plo = lo->l;
	int i = 1;
	while(plo != NULL){
	    auxe = dynarCreate(plo->e.x,plo->e.y,plo->e.z);
	    number = dynarLReadPos(lo,i,auxe);
	    dynarLAddLast(ld,number,auxe);
	    i++;
	    plo = plo->next;
	}
	return 1;
    }
    return -1;
}

/*

int main(){
    void *l = dynarLCreate();
    void *lcopy = dynarLCreate();
    int i,j,k;
    void *d1 = dynarCreate(1,1,1);
    void *d2 = dynarCreate(1,1,1);
    void *d3 = dynarCreate(1,1,1);
    
    dynarSetPos(d1,1,1,1,1);
    dynarSetPos(d2,2,1,1,1);
    dynarSetPos(d3,3,1,1,1);

    //printf("dynar1: %s\n",dynarString(d1));
    //printf("dynar2: %s\n",dynarString(d2));
    //printf("dynar3: %s\n",dynarString(d3));
    
    dynarLAddHNF(l,1,d1);
    dynarLAddHNF(l,2,d2);
    dynarLAddHNF(l,3,d3);

    //printf("dynarL: %s\n",dynarLString(l));

    //dynarLCopy(l,lcopy);

    //    dynarLRemovePos(l,3);

    dynarLReadNum(l,1,d1);
    //printf("dynar = %lf\n",dynarReadPos(d1,1,1,1));

    //printf("dynar = %lf\n",dynarLReadNumXYZ(l,1,1,1,1));

    
    //printf("dynarL: %s\n",dynarLString(l));
    ////printf("dynarLcopy: %s\n",dynarLString(lcopy));
    
        
    double newv1[2][2][2];
    double newv2[2][2][2];
    
    
    newv1[0][0][0] = 1;
    newv1[0][0][1] = 2;
    newv1[0][1][0] = 3;
    newv1[0][1][1] = 4;
    newv1[1][0][0] = 5;
    newv1[1][0][1] = 6;
    newv1[1][1][0] = 7;
    newv1[1][1][1] = 8;

    newv2[0][0][0] = 8;
    newv2[0][0][1] = 7;
    newv2[0][1][0] = 6;
    newv2[0][1][1] = 5;
    newv2[1][0][0] = 4;
    newv2[1][0][1] = 3;
    newv2[1][1][0] = 2;
    newv2[1][1][1] = 1;


    //printf("dynarLAdd\n");
    dynarLAddFirst(l,3,newv1,2,2,2);

    //printf("dynarLAdd\n");
    dynarLAddFirst(l,2,newv2,2,2,2);
    
    //printf("dynarLAdd\n");
    dynarLAddFirst(l,1,newv1,2,2,2);


    //printf("\n");
    for (i=0;i<2;i++)
	for(j=0;j<2;j++)
	    for(k=0;k<2;k++)
		//printf("val[%d][%d][%d] = %lf\n",i,j,k,dynarLReadPos(l,1,i+1,j+1,k+1));

    //printf("\n");
    for (i=0;i<2;i++)
	for(j=0;j<2;j++)
	    for(k=0;k<2;k++)
		//printf("val[%d][%d][%d] = %lf\n",i,j,k,dynarLReadPos(l,2,i+1,j+1,k+1)); 

    //printf("\n");
    for (i=0;i<2;i++)
	for(j=0;j<2;j++)
	    for(k=0;k<2;k++)
		//printf("val[%d][%d][%d] = %lf\n",i,j,k,dynarLReadPos(l,3,i+1,j+1,k+1));


    //printf("REMOVE ------ \n");
    dynarLRemove(l,3);

   //printf("\n");
    for (i=0;i<2;i++)
	for(j=0;j<2;j++)
	    for(k=0;k<2;k++)
		//printf("val[%d][%d][%d] = %lf\n",i,j,k,dynarLReadPos(l,1,i+1,j+1,k+1));

    //printf("\n");
    for (i=0;i<2;i++)
	for(j=0;j<2;j++)
	    for(k=0;k<2;k++)
		//printf("val[%d][%d][%d] = %lf\n",i,j,k,dynarLReadPos(l,2,i+1,j+1,k+1)); 

    //printf("\n");
    for (i=0;i<2;i++)
	for(j=0;j<2;j++)
	    for(k=0;k<2;k++)
		//printf("val[%d][%d][%d] = %lf\n",i,j,k,dynarLReadPos(l,3,i+1,j+1,k+1));
    
    

    //printf("dynarLDelete\n");
    dynarLDelete(l);
    
    return 1;
}

*/

#endif










