/**********************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the Modelica License 2
* Copyright © 2015, Victorino Sanz.
* 
***********************************************************/

#ifndef C17FHP
#define C17FHP

#include <CellularAutomataLib.c>


/****************************************/
/* LGCA MODELS (FHP) FOR ARGESIM C17 BENCHMARK      */
/****************************************/

//**************************************
// Cell state
// User may redefine the state of the cell as desired
typedef struct FHPState{
    // particles values 0=empty, 1= susceptible, 2= infected, 3= recovered
    //int nw,ne,w,e,sw,se; // particles 
    int p[6]; // particles {nw,ne,w,e,sw,se}
}FHPState;



//**************************************
// CELLULAR SPACE FUNCTIONS

// Function that allocates memory and sets the default state for the cells
void * FHPDefaultState(){
    FHPState *s;
    s = (FHPState *)malloc(sizeof(FHPState));
    int i;
    
    //set the default values for state variables
    for(i=0;i<6;i++)
	s->p[i] = 0;
    
    return (void *)s;
}

int FHPDefault(void* space){
    void*  (*defaultState)() = &FHPDefaultState;
    //CS2D_SetDefault(space, defaultState);
    CS2D_InitDefault(space, defaultState);
    return 1;
    
}

// Function that describes the state of initialized cells (non-initialized cells will remain in the default state)
void* InitialState(){
    FHPState *s;
    s = (FHPState *)malloc(sizeof(FHPState));
    
    //set the default values for state variables
    s->p[0] = 0;
    s->p[1] = 0;
    s->p[2] = 1;
    s->p[3] = 1; 
    s->p[4] = 0;
    s->p[5] = 0;
    
    return (void *)s;
}

// initializing function for the selected cell
void FHPInitial(void* space, int row, int col){
    void*  (*initialState)() = &InitialState;
    CS2D_Init(space, row, col, initialState);
    return;
}

void* RandomInitialState(){
    FHPState *s;
    s = (FHPState *)malloc(sizeof(FHPState));
    int i;

    //set the default values for state variables
    for(i=0;i<6;i++)
	s->p[i] = rand()%2;
    return (void *)s;
}

void FHPRandomInitial(void* space, int row, int col){
    int i,j;
    void*  (*initialState)() = &RandomInitialState;
    srand(time(NULL));
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++)
	    if (rand() % 2)
		CS2D_Init(space, i, j, initialState);
    return;
}

void* InitialS(){
    FHPState *s;
    s = (FHPState *)malloc(sizeof(FHPState));
    int i;

    for(i=0;i<6;i++)
	s->p[i] = 0;
    s->p[rand()%6] = 1;
    return (void *)s;
}

void* InitialI(){
    FHPState *s;
    s = (FHPState *)malloc(sizeof(FHPState));
    int i;

    for(i=0;i<6;i++)
	s->p[i] = 0;
    s->p[rand()%6] = 2;
    return (void *)s;
}

void* InitialSS(){
    FHPState *s;
    s = (FHPState *)malloc(sizeof(FHPState));
    int i;
    int s1,s2;

    s1 = rand()%6;
    s2 = rand()%6;
    while(s1 == s2)
	s2 = rand()%6;
    for(i=0;i<6;i++)
	if (i == s1 || i == s2)
	    s->p[i] = 1;
	else
	    s->p[i] = 0;
    return (void *)s;
}

void* InitialSSI(){
    FHPState *s;
    s = (FHPState *)malloc(sizeof(FHPState));
    int i;
    int s1,s2,i1;

    s1 = rand()%6;
    s2 = rand()%6;
    i1 = rand()%6;
    while(s1 == s2)
	s2 = rand()%6;
    while(i1 == s1 || i1 == s2)
	i1 = rand()%6;
    //set the default values for state variables
    for(i=0;i<6;i++)
	if (i == s1 || i == s2)
	    s->p[i] = 1;
	else if (i == i1)
	    s->p[i] = 2;
	else
	    s->p[i] = 0;
    
    return (void *)s;
}

void FHPSIRInitial(void* space, int row, int col){
    int i,j,n,pos,k;
    void*  (*defaultState)() = &FHPDefaultState;
    FHPState *s;
    int iniS = 16000;
    int iniI = 1000;
    int iniR = 0;
    
    srand(time(NULL));
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++)
	    CS2D_Init(space,i,j,defaultState);

    n = 0;
    while(n < iniS){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 1;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }

    n = 0;
    while(n < iniI){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 2;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }

    n = 0;
    while(n < iniR){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 1){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 3;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }
    
    return;
}

void FHPSIRInitial2(void* space, int row, int col){
    int i,j,n,pos,k;
    void*  (*defaultState)() = &FHPDefaultState;
    FHPState *s;
    int iniS = 40000;
    int iniI = 1000;
    int iniR = 0;
    
    srand(time(NULL));
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++)
	    CS2D_Init(space,i,j,defaultState);

    n = 0;
    while(n < iniS){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 1;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }

    n = 0;
    while(n < iniI){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 2;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }

    n = 0;
    while(n < iniR){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 1){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 3;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }
    
    return;
}



void FHPSIRInitialFull(void* space, int row, int col){
    int i,j,n,pos,k;
    void*  (*defaultState)() = &FHPDefaultState;
    FHPState *s;
    int iniS = 16000;
    int iniI = 100;
    int iniR = 4000;
    
    srand(time(NULL));
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++)
	    CS2D_Init(space,i,j,defaultState);


    n = 0;
    while(n < iniR){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 3;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }


    n = 0;
    while(n < iniI){
	i = rand()%row;
	j = rand()%(col/2);
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 2;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }


    n = 0;
    while(n < iniS){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 1;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }


    
    return;
}

void FHPSIRInitialHalf(void* space, int row, int col){
    int i,j,n,pos,k;
    void*  (*defaultState)() = &FHPDefaultState;
    FHPState *s;
    int iniS = 16000;
    int iniI = 100;
    int iniR = 4000;
    
    srand(time(NULL));
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++)
	    CS2D_Init(space,i,j,defaultState);


    n = 0;
    while(n < iniR){
	i = rand()%row;
	j = rand()%(col/2);
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 3;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }
    
    n = 0;
    while(n < iniI){
	i = rand()%row;
	j = rand()%(col/2);
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 2;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }


    n = 0;
    while(n < iniS){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 1;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }


    return;
}

void FHPSIRInitialBorder(void* space, int row, int col){
    int i,j,n,pos,k,starti,startj;
    void*  (*defaultState)() = &FHPDefaultState;
    FHPState *s;
    int iniS = 16000;
    int iniI = 100;
    int iniR = 4000;
    
    srand(time(NULL));
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++)
	    CS2D_Init(space,i,j,defaultState);


   n = 0;
    starti = 1;
    startj = 1;
    while(n < iniR){
	i = starti;
	for(j=startj;j<((col/2)-startj);j++){
	    s = CS2D_GetState(space,i,j);
	    for(pos=0;pos<6;pos++){
		if(s->p[pos] ==0){
		    s->p[pos] = 3;
		    n++;
		}
		if(n >= iniR)
		    break;
	    }
	    CS2D_SetState(space,i,j,s);
	}
	if(n >= iniR)
	    break;
	j = (col/2)-startj;
	for(i=starti;i<row-starti;i++){
	    s = CS2D_GetState(space,i,j);
	    for(pos=0;pos<6;pos++){
		if(s->p[pos] == 0){
		    s->p[pos] = 3;
		    n++;
		}
		if(n >= iniR)
		    break;
	    }
	    CS2D_SetState(space,i,j,s);
	}
	if(n >= iniR)
	    break;
	i = row-starti;
	for(j=startj;j<((col/2)-startj);j++){
	    s = CS2D_GetState(space,i,j);
	    for(pos=0;pos<6;pos++){
		if(s->p[pos] ==0){
		    s->p[pos] = 3;
		    n++;
		}
		if(n >= iniR)
		    break;
	    }
	    CS2D_SetState(space,i,j,s);
	}
	if(n >= iniR)
	    break;
	j = startj;
	for(i=starti;i<row-starti;i++){
	    s = CS2D_GetState(space,i,j);
	    for(pos=0;pos<6;pos++){
		if(s->p[pos] ==0){
		    s->p[pos] = 3;
		    n++;
		}
		if(n >= iniR)
		    break;
	    }
	    CS2D_SetState(space,i,j,s);
	}
	starti = starti+1;
	startj = startj+1;
    }
 
   n = 0;
    while(n < iniI){
	i = rand()%row;
	j = rand()%(col/2);
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 2;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }


    n = 0;
    while(n < iniS){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 1;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }

    //ModelicaFormatMessage("Vaccinated %d\n",n);
    return;
}


double uniform(double a, double b)
{
    return rand() / (RAND_MAX + 1.0) * (b - a) + a;
}

//***************************************************
// RULE FUNCTION
void *transition(int *change, void* cellstate, int row, int col, void** neighbors, int n_neighbors, void** inputs, int n_inputs, int *inputs_rcv){
    FHPState *out;
    FHPState *aux;
    FHPState *cs;
    FHPState **ne;
    double **in;
    double a = 0.2; // recovery rate
    double r = 0.6; // infection rate
    int i,j,I,N;
    double u;

    cs = (FHPState *)cellstate;
    ne = (FHPState **)neighbors;
    in = (double **)inputs;
    out = (FHPState *)malloc(sizeof(FHPState));
    aux = (FHPState *)malloc(sizeof(FHPState));
    
    for(i=0;i<6;i++)
	out->p[i] = 0; 
    


    // PROPAGATION
    for(i=0;i<n_neighbors;i++){
	if(neighbors[i] != NULL){ // NW
	    //ModelicaFormatMessage("SE (%d) from [%d,%d]\n",ne[0]->se,neighbors_present[0][0],neighbors_present[0][1]);
	    out->p[abs(i-(n_neighbors-1))] = ne[i]->p[abs(i-(n_neighbors-1))];
	}
    }
 
    // COLLISION
    //srand(time(NULL));
    // 2-collisions
    if (out->p[0] && out->p[5] && !out->p[1] && !out->p[2]  && !out->p[3] && !out->p[4]){
	//ModelicaFormatMessage("Collision NW-SE\n");
	// predefined orientation for collision
	out->p[1] = out->p[0];
	out->p[4] = out->p[5];
	// random orientation for collision
	/*if (rand() % 2){
	  out->p[1] = out->p[0];
	  out->p[4] = out->p[5];
	}else{
	  out->p[2] = out->p[0];
	  out->p[3] = out->p[5];
	  }*/
	out->p[0] = 0;
	out->p[5] = 0;
    }else if (out->p[1] && out->p[4] && !out->p[0] && !out->p[2]  && !out->p[3] && !out->p[5]){
	//ModelicaFormatMessage("Collision NE-SW\n");
	// predefined orientation for collision
	out->p[3] = out->p[1];
	out->p[2] = out->p[4];
	// random orientation for collision
	/*if (rand() % 2){
	    out->p[3] = out->p[1];
	    out->p[2] = out->p[4];
	  }else{
	    out->p[0] = out->p[1];
	    out->p[5] = out->p[4];
	    }*/
	out->p[1] = 0;
	out->p[4] = 0;	
    }else if (out->p[2] && out->p[3] && !out->p[0] && !out->p[1]  && !out->p[4] && !out->p[5]){
	//ModelicaFormatMessage("Collision W-E\n");
	// predefined orientation for collision
	out->p[0] = out->p[2];
	out->p[5] = out->p[3];
	// random orientation for collision
	/*if (rand() % 2){
	  out->p[0] = out->p[2];
	  out->p[5] = out->p[3];
	  }else{
	  out->p[1] = out->p[2];
	  out->p[4] = out->p[3];
	  }*/
	out->p[2] = 0;
	out->p[3] = 0;	
    // 3-collisions
    }else if (out->p[0] && out->p[3] && out->p[4] && !out->p[1]  && !out->p[2] && !out->p[5]){ 
	//ModelicaFormatMessage("Collision NW-E-SW\n");
	// predefined orientation for collision
	out->p[1] = out->p[0];
	out->p[5] = out->p[3];
	out->p[2] = out->p[4];
	// random orientation for collision
	/*if (rand() % 2){
	out->p[1] = out->p[0];
	out->p[5] = out->p[3];
	out->p[2] = out->p[4];
	}else{
	out->p[2] = out->p[0];
	out->p[1] = out->p[3];
	out->p[5] = out->p[4];
	    }*/
	out->p[0] = 0;
	out->p[3] = 0;	
	out->p[4] = 0;	
    }else if (out->p[1] && out->p[2] && out->p[5] && !out->p[0]  && !out->p[3] && !out->p[4]){ 
	//ModelicaFormatMessage("Collision NW-E-SW\n");
	// predefined orientation for collision
	out->p[3] = out->p[1];
	out->p[0] = out->p[2];
	out->p[4] = out->p[5];
	// random orientation for collision
	/*if (rand() % 2){
	out->p[3] = out->p[1];
	out->p[2] = out->p[2];
	out->p[4] = out->p[5];
	}else{
	out->p[0] = out->p[1];
	out->p[4] = out->p[2];
	out->p[3] = out->p[5];
	    }*/
	out->p[1] = 0;
	out->p[2] = 0;	
	out->p[5] = 0;	
    }


    

    // DISEASE SPREAD
    // Infection
    I = 0;
    for(i=0;i<6;i++)
	if(out->p[i] == 2)
	    I++;
    for(i=0;i<6;i++){
	u = uniform(0,1);
	if(I && out->p[i] == 1 && u <= r)//(1-pow(1-r,I)))
	    out->p[i] = 2;
    }
    
    // Recovery    
    for(i=0;i<6;i++){
	u = uniform(0,1);
	if(out->p[i] == 2 && u <= a)
	    out->p[i] = 3;
    }
    
    *change = 0;
    for(i=0;i<n_neighbors;i++)
       if(cs->p[i] != out->p[i])
	   *change = 1;

    return (void *)out;
}

// Function that returns the value to display in the graphical animation
double Display(void *cell){
    FHPState *s;
    double out;
    int i;

    s = (FHPState *)cell;
    // set out to the desired value calculated from the state variables
    if (cell == NULL)
	return 0;
    else{
	out = 0;
	for(i=0;i<6;i++)
	    if(s->p[i] == 2) // display infected
		out++;
	return out;
    }
}


int FHPStep(void* space){
    void* (*rule)(int*,void*,int,int,void**,int,void**,int,int*) = &transition;
    double (*display)(void*) = &Display;
    
    return CS2D_Step(space,rule,display);
    //return 1;
}

//***************************************************
// RULE FUNCTION
void *transitionR(int *change, void* cellstate, int row, int col, void** neighbors, int n_neighbors, void** inputs, int n_inputs, int *inputs_rcv){
    FHPState *out;
    FHPState *aux;
    FHPState *cs;
    FHPState **ne;
    double **in;
    double a = 0.2; // recovery rate
    double r = 0.3; // infection rate
    int i,j,I,N;
    double u;

    cs = (FHPState *)cellstate;
    ne = (FHPState **)neighbors;
    in = (double **)inputs;
    out = (FHPState *)malloc(sizeof(FHPState));
    aux = (FHPState *)malloc(sizeof(FHPState));
    
    for(i=0;i<6;i++)
	out->p[i] = 0; 
    


    // PROPAGATION
    for(i=0;i<n_neighbors;i++){
	if(neighbors[i] != NULL){ // NW
	    //ModelicaFormatMessage("SE (%d) from [%d,%d]\n",ne[0]->se,neighbors_present[0][0],neighbors_present[0][1]);
	    out->p[abs(i-(n_neighbors-1))] = ne[i]->p[abs(i-(n_neighbors-1))];
	}
    }
 
    // COLLISION
    //srand(time(NULL));
    // 2-collisions
    if (out->p[0] && out->p[5] && !out->p[1] && !out->p[2]  && !out->p[3] && !out->p[4]){
	//ModelicaFormatMessage("Collision NW-SE\n");
	// predefined orientation for collision
	out->p[1] = out->p[0];
	out->p[4] = out->p[5];
	// random orientation for collision
	/*if (rand() % 2){
	  out->p[1] = out->p[0];
	  out->p[4] = out->p[5];
	}else{
	  out->p[2] = out->p[0];
	  out->p[3] = out->p[5];
	  }*/
	out->p[0] = 0;
	out->p[5] = 0;
    }else if (out->p[1] && out->p[4] && !out->p[0] && !out->p[2]  && !out->p[3] && !out->p[5]){
	//ModelicaFormatMessage("Collision NE-SW\n");
	// predefined orientation for collision
	out->p[3] = out->p[1];
	out->p[2] = out->p[4];
	// random orientation for collision
	/*if (rand() % 2){
	    out->p[3] = out->p[1];
	    out->p[2] = out->p[4];
	  }else{
	    out->p[0] = out->p[1];
	    out->p[5] = out->p[4];
	    }*/
	out->p[1] = 0;
	out->p[4] = 0;	
    }else if (out->p[2] && out->p[3] && !out->p[0] && !out->p[1]  && !out->p[4] && !out->p[5]){
	//ModelicaFormatMessage("Collision W-E\n");
	// predefined orientation for collision
	out->p[0] = out->p[2];
	out->p[5] = out->p[3];
	// random orientation for collision
	/*if (rand() % 2){
	  out->p[0] = out->p[2];
	  out->p[5] = out->p[3];
	  }else{
	  out->p[1] = out->p[2];
	  out->p[4] = out->p[3];
	  }*/
	out->p[2] = 0;
	out->p[3] = 0;	
    // 3-collisions
    }else if (out->p[0] && out->p[3] && out->p[4] && !out->p[1]  && !out->p[2] && !out->p[5]){ 
	//ModelicaFormatMessage("Collision NW-E-SW\n");
	// predefined orientation for collision
	out->p[1] = out->p[0];
	out->p[5] = out->p[3];
	out->p[2] = out->p[4];
	// random orientation for collision
	/*if (rand() % 2){
	out->p[1] = out->p[0];
	out->p[5] = out->p[3];
	out->p[2] = out->p[4];
	}else{
	out->p[2] = out->p[0];
	out->p[1] = out->p[3];
	out->p[5] = out->p[4];
	    }*/
	out->p[0] = 0;
	out->p[3] = 0;	
	out->p[4] = 0;	
    }else if (out->p[1] && out->p[2] && out->p[5] && !out->p[0]  && !out->p[3] && !out->p[4]){ 
	//ModelicaFormatMessage("Collision NW-E-SW\n");
	// predefined orientation for collision
	out->p[3] = out->p[1];
	out->p[0] = out->p[2];
	out->p[4] = out->p[5];
	// random orientation for collision
	/*if (rand() % 2){
	out->p[3] = out->p[1];
	out->p[2] = out->p[2];
	out->p[4] = out->p[5];
	}else{
	out->p[0] = out->p[1];
	out->p[4] = out->p[2];
	out->p[3] = out->p[5];
	    }*/
	out->p[1] = 0;
	out->p[2] = 0;	
	out->p[5] = 0;	
    }


    

    // DISEASE SPREAD
    // Infection
    I = 0;
    for(i=0;i<6;i++)
	if(out->p[i] == 2)
	    I++;
    for(i=0;i<6;i++){
	u = uniform(0,1);
	if(I && out->p[i] == 1 && u <= r)// (1-pow(1-r,I)))
	    out->p[i] = 2;
    }
    
    // Recovery    
    for(i=0;i<6;i++){
	u = uniform(0,1);
	if(out->p[i] == 2 && u <= a)
	    out->p[i] = 3;
    }
    
    *change = 0;
    for(i=0;i<n_neighbors;i++)
       if(cs->p[i] != out->p[i])
	   *change = 1;

    return (void *)out;
}


void FHPSIRRearrange(void* space, int row, int col){
    int i,j,n,pos,k;
    void*  (*defaultState)() = &FHPDefaultState;
    FHPState *s;
    int iniS = 0;
    int iniI = 0;
    int iniR = 0;
    
    
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++){
	    s = CS2D_GetState(space,i,j);
	    for(pos=0;pos<6;pos++){
		if(s->p[pos] == 1)
		    iniS++;
		else if(s->p[pos] == 2)
		    iniI++;
		else if(s->p[pos] == 3)
		    iniR++;
	    }
	}
    
    srand(time(NULL));
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++)
	    CS2D_Init(space,i,j,defaultState);

    n = 0;
    while(n < iniS){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 1;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }

    n = 0;
    while(n < iniI){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 2;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }

    n = 0;
    while(n < iniR){
	i = rand()%row;
	j = rand()%col;
	s = CS2D_GetState(space,i+1,j+1);
	k=1;
	pos = rand()%6;
	while(k<7 && s->p[pos] != 0){
	    pos = (pos+1)%6;
	    k++;
	}
	if (k < 7){
	    s->p[pos] = 3;
	    n++;
	}
	CS2D_SetState(space,i+1,j+1,s);
    }
    
    return;
}

int FHPStepR(void* space){
    void* (*rule)(int*,void*,int,int,void**,int,void**,int,int*) = &transitionR;
    double (*display)(void*) = &Display;
    
    CS2D_Step(space,rule,display);
    FHPSIRRearrange(space,100,100);
    return 1;
}



/*********************************************/

//**************************************
// INTERFACE FUNCTIONS

// function to convert the state of a cell into a double value
double OutputS(void* space, int row, int col){
    double out;
    FHPState* s;
    int i;
    
    s = (FHPState *)CS2D_GetState(space,row,col);
    
    out = 0;
    for(i=0;i<6;i++)
	if (s->p[i] == 1)
	    out++;
    
    return out;
}

// function to convert the state of a cell into a double value
double OutputI(void* space, int row, int col){
    double out;
    FHPState* s;
    int i;
    
    s = (FHPState *)CS2D_GetState(space,row,col);
    out = 0;
    for(i=0;i<6;i++)
	if (s->p[i] == 2)
	    out++;
	
    return out;
}

// function to convert the state of a cell into a double value
double OutputR(void* space, int row, int col){
    double out;
    FHPState* s;
    int i;

    s = (FHPState *)CS2D_GetState(space,row,col);
    out = 0;
    for(i=0;i<6;i++)
	if (s->p[i] == 3)
	    out++;
    
    return out;
}


// function to set an external value as an input to a cell
void VFull(void* space,int row, int col, double value,int input_id){
    double* inp;

    inp = (double *)malloc(sizeof(double));
    //convert the external value to values of the state variables
    *inp = value;
    if (mod(row,2) == 0 && mod(col,1) == 0)
	*inp = value;
    else
	*inp = -1;
    CS2D_SetInput(space,row,col,(void*)inp,input_id);

    return;
}

void FHPInitS1(void* space, int row, int col, double value){
    FHPState* s;
    int pos;

    s = CS2D_GetState(space,row,col);
    if(mod(row-1,2) == 0 && mod(col-1,1) == 0){
	pos = rand()%6;
	while(s->p[pos] != 0)
	    pos = rand()%6;
	s->p[pos] = 1; 
	
	while(s->p[pos] != 0)
	    pos = rand()%6;
	s->p[pos] = 1; 
	
    }else{
	pos = rand()%6;
	while(s->p[pos] != 0)
	    pos = rand()%6;
	s->p[pos] = 1; 
    }
    CS2D_SetState(space,row,col,s);
    return;
}

void FHPInitS2(void* space, int row, int col, double value){
    FHPState* s;
    int pos;

    s = CS2D_GetState(space,row,col);
    pos = rand()%6;
    while(s->p[pos] != 0)
	pos = rand()%6;
    s->p[pos] = 1; 
    CS2D_SetState(space,row,col,s);
    return;
}

void FHPInitI1(void* space, int row, int col, double value){
    FHPState* s;
    int pos;

    s = CS2D_GetState(space,row,col);
    if(mod(row-1,10) == 0 && mod(col-1,10) == 0){
	pos = rand()%6;
	while(s->p[pos] != 0)
	    pos = rand()%6;
	s->p[pos] = 2; 
    }
    CS2D_SetState(space,row,col,s);
    return;
}

void FHPInitI2(void* space, int row, int col, double value){
    FHPState* s;
    int pos;

    s = CS2D_GetState(space,row,col);
    if(mod(row-1,10) == 0 && mod(col-1,10) == 0){
	pos = rand()%6;
	while(s->p[pos] != 0)
	    pos = rand()%6;
	s->p[pos] = 2; 
	pos = rand()%6;
	while(s->p[pos] != 0)
	    pos = rand()%6;
	s->p[pos] = 2; 
    }
    CS2D_SetState(space,row,col,s);
    return;
}

void FHPInitRfull(void* space, int row, int col, double value){
    FHPState* s;
    int pos;

    s = CS2D_GetState(space,row,col);
    if(mod(row-1,2) == 0 && mod(col-1,1) == 0){
	pos = rand()%6;
	while(s->p[pos] != 0)
	    pos = rand()%6;
	s->p[pos] = 3; 

    }
    CS2D_SetState(space,row,col,s);
    return;
}

void FHPInitRhalf(void* space, int row, int col, double value){
    FHPState* s;
    int pos;

    s = CS2D_GetState(space,row,col);
    pos = rand()%6;
    while(s->p[pos] != 0)
	pos = rand()%6;
    s->p[pos] = 3; 
    
    CS2D_SetState(space,row,col,s);
    return;
}


void FHPInitRborder(void* space, int row, int col, double value){
    FHPState* s;
    int pos;

    s = CS2D_GetState(space,row,col);
    for(pos=0;pos<6;pos++)
	if(s->p[pos] == 0)
	    s->p[pos] = 3;
 
     CS2D_SetState(space,row,col,s);
    return;
}

#endif
