/**********************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the Modelica License 2
* Copyright © 2013, Victorino Sanz.
* 
***********************************************************/

#ifndef DRAFT
#define DRAFT

#include <CellularAutomataLib.c>


/****************************************/
/* DRAFT CELLULAR AUTOMATA       */
/****************************************/

//**************************************
// Cell state
// User may redefine the state of the cell as desired
typedef struct State{
    double t;
}State;



//**************************************
// CELLULAR SPACE FUNCTIONS

// Function that allocates memory and sets the default state for the cells
void * DefaultState(){
    State *s;
    s = (State *)malloc(sizeof(State));
    
    //set the default values for state variables
    s->t = 0;
    return (void *)s;
}

// function to set the default state function in the space
// USED TO REDECLARE Default in CellSpace model
int Default(void *state){
    void* (*defaultState)() = &DefaultState;
    return CS2D_InitDefault(state,defaultState);
}


// Function that allocates memory and sets the state of initialized cells (non-initialized cells will be created using the default state)
void* InitialState(){
    State *s;
    
    s = (State *)cell;
    // set the values of state variables for initialized cells
    s->t = 1.0;
    
    return (void*)s;
}

// initializing function for the selected cell
// USED TO REDECLARE Initial in CellSpace model
void Initial(int space, int row, int col){
    void  (*initialState)(void *) = &InitialState;
    CS2D_Init(space, row, col, initialState);
    return;
}

//***************************************************
// RULE FUNCTION
void *transition(int* change, void* cellstate, int row, int col, void** neighbors, int n_neighbors, void** inputs, int n_inputs, int *inputs_rcv){
    State *out;
    State *cs;
    State **ne;
    State **in;
    
    cs = (State *)cellstate;
    ne = (State **)neighbors;
    in = (State **)inputs;
    out = (State *)malloc(sizeof(State));
    
    // define the behavior of the model during transitions

    *change = 0;
    if(cs->t != out->t)
	*change = 1;
    
    return (void *)out;
}

// Function that returns the value to display in the graphical animation
double Display(void *cell){
    State *s;
    double out;

    s = (State *)cell;
    // set out to the desired value calculated from the state variables
    out = s->t;

    return out;
}

// function used to set the functions used in the simulation step
// USED TO REDECLARE Step FUNCTION IN CellSpace MODEL
int Step(int space){
    void* (*rule)(int*,void*,int,in void**,int,void**,int,int*) = &transition;
    double (*display)(void*) = &Display;
    
    CS2D_Step(space,rule,display);
    return 1;
}

/*********************************************/

//**************************************
// INTERFACE FUNCTIONS


// function initialize a cell using an external value
void ExtInit(void* space,int row, int col, double value){
    State* s;

    s = (State *)CS2D_Get(space,row,col);
    // assign external value to the state variables
    s->t = value;
    CS2D_Init(space,row,col,NULL);
    return;
}


// Function to set a new input to a cell, from other cellular space
void SetInput(void* Fspace, int Frow, int Fcol, void* Tspace, int Trow, int Tcol, int input_id){
    State* s;
    State* inp;
    
    inp = (State *)malloc(sizeof(State));
    s = (State *)CS2D_GetState(Fspace,Frow,Fcol);
    *inp = *s;
    CS2D_SetInput(Tspace,Trow,Tcol,(void*)inp,input_id);
}


// function to set an external value as an input to a cell
void ExtInput(void* space,int row, int col, double value,int input_id){
    State* inp;

    inp = (State *)malloc(sizeof(State));
    //convert the external value to values of the state variables
    inp->t = value;

    CS2D_SetInput(space,row,col,(void*)inp,input_id);

    return;
}

// function to convert the state of a cell into a double value
double Output(void* space, int row, int col){
    double out;
    
    out = Display(CS2D_Get(space,row,col));
    
    return out;
}

#endif
