/**********************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the Modelica License 2
* Copyright © 2013, Victorino Sanz.
* 
***********************************************************/

#ifndef ENVIRONMENT
#define ENVIRONMENT

typedef struct freeID{
    int id;
    struct freeID *n; // next
    struct freeID *p; // prev
} freeID;

int IDput (freeID **l, int id){
    freeID *f;
    freeID *aux;
    
    f = (freeID*) malloc(sizeof(freeID));
    f->id = id;
    f->n = NULL;
    f->p = NULL;
    //    printf("new freeid = %p, id = %d\n",f,f->id);

    
    if ((*l) == NULL){
	//	printf("empty list %p\n",l);
	(*l) = f;
	//printf("one element list %p\n",f);
    }else{
	aux = (*l);
	if(aux->id > id){ // insert first
	    f->n = aux;
	    aux->p = f;
	    (*l) = f;
	}else{ // find place
	    aux = (*l);
	    while (aux->n != NULL && aux->n->id < id)
		aux = aux->n;
	    if(aux->n != NULL){ // middle of the list
		f->n = aux->n;
		f->p = aux;
		aux->n->p = f;
		aux->n = f;
	    }else{ // insert at the end of the list
		aux->n = f;
		f->p = aux;
	    }
	}
    }
    return 1;
}


int IDget (freeID **l){
    freeID *f;
    freeID *aux;
    int id;
    
    if ((*l) != NULL){
	aux = (*l);
	f = aux;
	(*l) = aux->n;
	f->n = NULL;
	f->p = NULL;
	id = f->id;
	free(f);
	if ((*l) == NULL)
	    IDput(l,id+1); // if empty list, add next id
    }else
	id = -1;
    return id;
}


/**********************************************************************************/

typedef struct AGENT{
    int section; // section of the environment where the agent is located
    int id; // agent ID
    int reinit; // reinit flag for continuous-time dynamics of the agent
    double order; // value for ordering agents in the population list
    double x; // x coordinate for the agent
    double y; // y coordinate for the agent
    int nattr; // number of attributes
    double *attr; // agent attributes
    struct AGENT *next; // next agent in the pop
    struct AGENT *prev; // previous agent in the pop
} AGENT;

typedef struct POP{
    int popsize; // population size
    AGENT* l; // list of agents
    int refmax; // maximum agent ID in the population
    int initags; // initialization of agent references
    AGENT** ref; // list of direct references to agents (indexed by agent ID)
} POP;

typedef struct CELL{ // cell for discrete environment
    // int x; // x coordinate
    //int y; // y coordinate
    int nagents; // number of agents of the cell
    int maxnagents; // max size of agids
    int *agids; // IDs of agents present in the cell
    double *vars; // variables of the cell
} CELL;

typedef struct SEC{
    int number;
    CELL **cells; // cells for discrete environments
    int X; // X dimension of the environment
    int Y; // Y dimension of the environment
    int nSecVars; // number of section variables
    double *SecVars; // section variables
    int nvars; // number of variables per cell
    int plot_var; // variable to plot in the animation
    FILE **gp; // file descriptor to communicate with gnuplot
    char *path; // path to gnuplot
    int animation; // boolean to control graphical animation
} SEC;

typedef struct ENV{
    freeID **freeids; // next ID free to be assigned to an agent
    int nsections; // number of sections in the environment
    SEC **sections; // array of pointers to sections
    POP *population; // population of agents in the environment
} ENV;

void *ENVCreate(int maxinitags){
    int i;
    ENV *e;

    printf("ENV Create\n");
    // Create empty environment (sections should be added later)
    e = (ENV*)malloc(sizeof(ENV));
    e->nsections = 0;
    e->sections = NULL;
    e->freeids = (freeID**)malloc(sizeof(freeID*));
    e->freeids[0] = NULL;
    printf("------------ freeids = %p\n",e->freeids);
    IDput(e->freeids,1);
    printf("============ freeids = %p\n",e->freeids);
    // Create POPULATION
    e->population = (POP*)malloc(sizeof(POP));
    e->population->popsize = 0;
    e->population->l = NULL;
    e->population->refmax = maxinitags;
    e->population->ref = (AGENT**)malloc(maxinitags*sizeof(AGENT*));
    for(i=0;i<maxinitags;i++)
	e->population->ref[i] = NULL;

    printf("ENVIRONMENT = %p\n",e);
    return (void *)e;
}

int ENVDelete(void *environment){
    ENV *e = (ENV*)environment;

    free(e->sections);
    free(e->population);
    return 1;
}

int ENVaddsec(void *environment, void* section, int secnumber){
    ENV *e = (ENV*)environment;
    SEC *s = (SEC*)section;

    if (secnumber > e->nsections){
	e->nsections = secnumber;
	e->sections = (SEC**)realloc(e->sections,e->nsections*sizeof(SEC*));
    }
    e->sections[secnumber-1] = s;
    return 1;
}

int newID(void *environment,int id){
    ENV*e = (ENV*)environment;

    //    printf("newID ");
    if (id < 1)
	return (IDget(e->freeids));
    else
	IDput(e->freeids,id);
    return 0;
}

void *SECCreate(int number,int x, int y, int nSecVars, int nvars, int plot_var, const char *path, const char *name, int plot_range, int plot_animation){
    SEC *s;
    int i,j,k,n;


    printf("CreateSEC %d x %d \n",x,y);
    s = (SEC*)malloc(sizeof(SEC));
    s->number = number;
    s->X = x;
    s->Y = y;
    //    printf("s= %p, x=%d, y= %d, X = %d, Y = %d \n",s,x,y,s->X,s->Y);
    s->nSecVars = nSecVars;
    s->SecVars = (double*)malloc(nSecVars*sizeof(double));
    for (i=0;i<nSecVars;i++)
	s->SecVars[i] = 0;
    s->nvars = nvars;
    s->plot_var = plot_var-1;
    s->animation = plot_animation;


    // CELLS
    s->cells = (CELL**)malloc(x*sizeof(CELL*));
    for(i=0;i<x;i++)
	s->cells[i] = (CELL*)malloc(y*sizeof(CELL));
    for(i=0;i<x;i++)
	for(j=0;j<y;j++){
	    //s->cells[i][j].x = i;
	    //s->cells[i][j].y = j;
	    s->cells[i][j].nagents = 0; // initially no agents in the cell
	    s->cells[i][j].maxnagents = 10;
	    s->cells[i][j].agids = (int*)malloc(s->cells[i][j].maxnagents*sizeof(int));
	    //for(k=0;k<s->cells[i][j].maxnagents;k++)
	    //	s->cells[i][j].agids[k] = 0;
	    s->cells[i][j].vars = (double*)malloc(nvars*sizeof(double));
	    for (n=0;n<nvars;n++)
		s->cells[i][j].vars[n] = 0; // default value for cell variables
	}
    s->path = (char*)malloc(strlen(path)+1*sizeof(char));
    strcpy(s->path,path);
    
    if (plot_animation) {
	s->gp = (FILE**)malloc(nvars*sizeof(FILE*));
	
#ifdef _WIN32
	s->gp[s->plot_var] = _popen(path, "w");
#else
	s->gp[s->plot_var] = popen(path, "w");
#endif
	
	if(s->gp[s->plot_var] == NULL){
	    fprintf(stderr, "Oops, I can't find %s.", path);
	    exit(EXIT_FAILURE);
	}    
#ifdef _WIN32
	fprintf(s->gp[s->plot_var], "set terminal wxt persist \n");
#else		  
	fprintf(s->gp[s->plot_var], "set terminal x11\n");
#endif		  
	fprintf(s->gp[s->plot_var], "set title \"%s\" \n",name);
	fprintf(s->gp[s->plot_var], "set xrange [0:%d] \n",x);
	fprintf(s->gp[s->plot_var], "set yrange [0:%d] \n",y);
	fprintf(s->gp[s->plot_var], "set autoscale xy \n");
	fprintf(s->gp[s->plot_var], "set cbrange [0:%d] \n",plot_range);	    
    }
    printf("SECTION = %p\n",s);
    return (void *)s;
}

int SECDelete(void *section){
    SEC *s = (SEC*)section;
    int i;

    printf("DeleteSEC\n");

    /*  for (i=0;i<e->pop->popsize;i++)
	free(e->pop->ref[i]);
    if (e->pop->l != NULL)
	free(e->pop->l);
    if (e->pop->ref != NULL)
	free(e->pop->ref);
    */
    free(s->cells);
    return 1;
}

int secnumber(void *section){
    SEC *s = (SEC*)section;

    if (s != NULL)
	return s->number;
    else
	return -1;
}

int SECVARset(void *section, int mvar, double newvalue){
    SEC*s = (SEC*)section;
    int i;
    int var = mvar -1;

    printf("SECVARSet %p, var %d = %lf \n",section,mvar,newvalue );
    s->SecVars[var] = newvalue;
    return 1;
}

int ENV_SECVARset(void *environment, int agid, int mvar, double newvalue){
    ENV *e=(ENV*) environment;
    POP *p;
    int sindex;
    SEC *s;
    int index = agid-1;

    printf("ENV_SECVARSet %p, agent %d, var %d = %lf \n",environment, agid, mvar,newvalue );
    if (e != NULL){
	p = e->population;
	sindex = p->ref[index]->section-1; // current section
	s = e->sections[sindex];
	SECVARset((void*)s,mvar,newvalue);
    }
    return 1;
}


double SECVARread(void *section, int mvar){
    SEC*s = (SEC*)section;
    int var = mvar -1;

    printf("SEC_VARREAD %d\n",mvar);
    return s->SecVars[var];
       
    return 1;
}

int ENV_SECVARread(void *environment, int agid, int mvar){
    ENV *e=(ENV*) environment;
    POP *p;
    int sindex;
    SEC *s;
    int i;
    int var = mvar -1;
    int index = agid-1;

    printf("ENV_SECVARread %p, agent %d, var %d \n",environment, agid, mvar);
    if (e != NULL){
	p = e->population;
	printf("POP = %p\n",p); 
	sindex = p->ref[index]->section-1; // current section
	printf("sindex = %d\n",sindex);
	s = e->sections[sindex];
	printf("section = %p\n",s);
	return SECVARread((void*)s,mvar);
    }
    return -1;
}


int addplot(void *section, int plot_var, int plot_range, const char *name){
    SEC *s = (SEC*)section;
    int var = plot_var-1;
    

    printf("addplot \n");
    if (s->animation){
#ifdef _WIN32
	s->gp[var] = _popen(s->path, "w");
#else
	s->gp[var] = popen(s->path, "w");
#endif
	
	if(s->gp[var] == NULL){
	    fprintf(stderr, "Oops, I can't find %s.", s->path);
	    exit(EXIT_FAILURE);
	}    
#ifdef _WIN32
	fprintf(s->gp[var], "set terminal wxt persist \n");
#else
	fprintf(s->gp[var], "set terminal x11 \n");
#endif
	fprintf(s->gp[var], "set title \"%s\" \n",name);
	fprintf(s->gp[var], "set xrange [0:%d] \n",s->X);
	fprintf(s->gp[var], "set yrange [0:%d] \n",s->Y);
	fprintf(s->gp[var], "set autoscale xy \n");
	fprintf(s->gp[var], "set cbrange [0:%d] \n",plot_range);
    }
    
    return 1;    
}



// Function that describes the state of initialized cells (non-initialized cells will remain in the default state)
int plot(void *section,int plot_var){
    SEC*s = (SEC*)section;
    int i,j;
    int var;
    double *envplot;
    
        printf("PLOT s=%p\n",s);
    if (s->animation){
	if (plot_var <= 0)
	    var = 0;
	else
	    var = plot_var-1;
	//printf("var= %d\n",var);
	fprintf(s->gp[var],"plot \'-\' matrix with image\n");
	for(j=0;j<s->Y;j++){
	    for(i=0;i<s->X;i++){
		fprintf(s->gp[var],"%f ",s->cells[i][j].vars[var]);
		//	printf("%f ",s->cells[i][j].vars[var]);
	    }
	    fprintf(s->gp[var],"\n ");
	    //printf("\n ");
	}
	fprintf(s->gp[var],"\n");
	//printf("\n");
	fprintf(s->gp[var],"e\n");
	fflush(s->gp[var]);
    }
    //    printf("end plot\n");
    return 1;
}

int clear(void *section, int mvar){
    SEC*s = (SEC*)section;
    int i,j;
    int var = mvar-1;
  
    printf("CLEAR\n");
    for(i=0;i<s->X;i++){
	for(j=0;j<s->Y;j++){
	    s->cells[i][j].vars[var] = 0;
	}
    }
    return 1;
}

int VARmodify(void *section, int posx, int posy, int mvar, double modifier){
    SEC*s = (SEC*)section;
    int x = posx-1;
    int y = posy-1;
    int i;
    int var = mvar -1;

        printf("VARMODIFY s= %p, mvar=%d [%d,%d]\n",s,mvar,posx,posy);
    x = posx < 1 ? 0 : posx-1;
    y = posy < 1 ? 0 : posy-1;
    //printf("x=%d, y= %d, var = %d , X = %d, Y = %d \n",x,y,var,s->X,s->Y);
    x = x > s->X-1 ? s->X-1 : x;
    y = y > s->Y-1 ? s->Y-1 : y;
    //printf("MOD cells[%d][%d].vars[%d] = %lf, modifier = %lf \n",x,y,var,s->cells[x][y].vars[var],modifier);
    s->cells[x][y].vars[var] += modifier;
    //printf("MOD cells[%d][%d].vars[%d] = %lf \n",x,y,var,s->cells[x][y].vars[var]);
    
    return 1;
}

int VARset(void *section, int posx, int posy, int mvar, double newvalue){
    SEC*s = (SEC*)section;
    int x = posx-1;
    int y = posy-1;
    int i;
    int var = mvar -1;

    printf("VARSet %p [%d,%d], var %d = %lf \n",section,posx,posy,mvar,newvalue );
    x = posx < 1 ? 0 : posx-1;
    y = posy < 1 ? 0 : posy-1;
    x = x > s->X-1 ? s->X-1 : x;
    y = y > s->Y-1 ? s->Y-1 : y;
    printf("SET cells[%d][%d].vars[%d] = %lf, newvalue = %lf \n",x,y,var,s->cells[x][y].vars[var],newvalue);
    s->cells[x][y].vars[var] = newvalue;
    printf("SET cells[%d][%d].vars[%d] = %lf\n",x,y,var,s->cells[x][y].vars[var]);
    return 1;
}

double VARread(void *section, int posx, int posy, int mvar){
    SEC*s = (SEC*)section;
    int x = posx-1;
    int y = posy-1;
    int var = mvar -1;

    printf("VARREAD %d\n",mvar);
    x = posx < 1 ? 0 : posx-1;
    y = posy < 1 ? 0 : posy-1;
    x = x > s->X-1 ? s->X-1 : x;
    y = y > s->Y-1 ? s->Y-1 : y;
    //printf("READ cells[%d][%d].vars[%d] = %lf \n",x,y,var,s->cells[x][y].vars[var]);
    return s->cells[x][y].vars[var];
    //printf("READ cells[%d][%d].vars[%d] = %lf \n",x,y,var,s->cells[x][y].vars[var]);
    
    return 1;
}


//***********************************************************************************************
//***********************************************************************************************
//***********************************************************************************************
//***********************************************************************************************

int popsize(void *environment){
    ENV*e = (ENV*)environment;
    POP*p = e->population;
    
    //    printf("POPSIZE \n");
    return p->popsize;
    
}

int AGinsert(void *environment, int section, int agid, double xpos, double ypos, double order, double *agattr, size_t nattr){
    ENV *e = (ENV*)environment;
    POP *p = e->population;
    SEC *s;
    AGENT *newag;
    AGENT *pa;
    int i;
    int index = agid-1;
    int sindex = section-1;
    int cellx;
    int celly;

        printf("INSERT_AG (env %p) %d in section %d [%lf,%lf], with order %lf \n",environment,agid,section,xpos,ypos, order);
  //  printf("population size %d, refmax %d \n",p->popsize,p->refmax);
  //  pa=p->l;
  //  for (i=0;i<p->popsize;i++){
  //	printf("%d,",pa->id);
  //	pa=pa->next;
  //  }
  // printf("\n");

    if (p != NULL && section <= e->nsections){
	s = e->sections[sindex];
	cellx = xpos < 0 ? 0 : xpos;
	celly = ypos < 0 ? 0 : ypos;
	cellx = cellx >= s->X ? s->X-1 : cellx;
	celly = celly >= s->Y ? s->Y-1 : celly;
	//printf("cellx = %d, celly = %d\n",cellx,celly);
	if (p->ref[index] == NULL){ // new agent
	    newag = (AGENT*)malloc(sizeof(AGENT));
	    newag->section = section;
	    newag->id = agid;
	    newag->reinit = 0;
	    newag->order = order;
	    newag->x = xpos;
	    newag->y = ypos;
	    newag->nattr = nattr;
	    newag->attr = (double*)malloc(nattr*sizeof(double));
	    for(i=0;i<nattr;i++) // assign attributes
		newag->attr[i] = agattr[i];
	    if(agid >= p->refmax){ // update list of direct references to agents
		p->ref = (AGENT**)realloc(p->ref,2*agid*sizeof(AGENT*));   
		for(i=p->refmax-1;i<2*agid;i++)
		    p->ref[i] = NULL;
		p->refmax = 2*agid;
	    }
	    newag->next = NULL;
	    newag->prev = NULL;
	    p->popsize++;
	    //printf("new agent created %p, index %d, popsize %d\n",newag,index,p->popsize);
	}else{ // agent already exists
	    ModelicaFormatMessage("AGENT %d ALREADY IN THE POPULATION!\n",agid); 
	    return -1;
	}
	//printf("nagents = %d\n",s->cells[cellx][celly].nagents);
	s->cells[cellx][celly].nagents++;
	if (s->cells[cellx][celly].nagents >= s->cells[cellx][celly].maxnagents){
	    s->cells[cellx][celly].maxnagents*=2;
	    s->cells[cellx][celly].agids = (int*)realloc(s->cells[cellx][celly].agids,s->cells[cellx][celly].maxnagents*sizeof(int));
	}
	s->cells[cellx][celly].agids[s->cells[cellx][celly].nagents-1] = newag->id;
	p->ref[index] = newag;
	pa = p->l;
	//printf("insert in list %p\n",pa);
	if (pa == NULL || pa->order > order){ // empty population or insert first
	    //printf("insert first %lf\n",order);
	    newag->next = pa;
	    if (pa != NULL)
		pa->prev = newag;
	    newag->prev = NULL;
	    p->l = newag;
	}else{ // find the insertion place
	    //printf("find place %lf for agent %p\n",order,newag);
	    newag->prev = pa;
	    //printf("next = %p\n",pa->next);
	    pa = pa->next;
	    while(pa != NULL){
		//printf("pa = %p\n",pa);
	        if(pa->order > order){ // insert before pa
		    //printf("insert before pa \n");
		    newag->next = pa;
		    newag->prev = pa->prev;
		    pa->prev->next = newag;
		    pa->prev = newag;
		    //printf("inserted...\n");
		    return 1;
		}else{ // check next
		    //printf("check next of %d \n",newag->id);
		    //printf("pa %p \n",pa);
		    //printf("pa->next %p \n",pa->next);
		    //printf("newag %p \n",newag);
		    //printf("newag->prev %p \n",newag->prev);
		    newag->prev = pa;
		    //printf("advance prev \n");
		    pa = pa->next;
		    //printf("next = %p\n",pa->next);
		}
	    } // insert at the end;
	    //	    printf("1 id = %d, newag = %p, prev = %p, next = %p\n",newag->id,newag,newag->prev,newag->next);
	    newag->next = NULL;
	    newag->prev->next = newag;
	}
	//printf("id = %d, newag = %p, prev = %p, next = %p\n",newag->id,newag,newag->prev,newag->next);
    }
    //     printf("inserted...\n");
    return 1;
}

int AGupdate(void *environment, int section, int agid, double xpos, double ypos, double *attr, size_t nattr){
    ENV *e = (ENV*)environment;
    SEC *ps; // previous section
    SEC *s; // new section
    POP *p = e->population;
    AGENT *newag;
    AGENT *pa;
    int i,n,k;
    int index = agid-1;
    int sindex = section -1;
    int psindex;
    int newx;
    int newy;
    int prex, prey;

    printf("UPDATE_AG id=%d, xpos = %lf, ypos = %lf, \n",agid,xpos,ypos);
    if (p != NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){ // agent present in the population
	    psindex = p->ref[index]->section-1; // current section
	    ps = e->sections[psindex];
	    prex = p->ref[index]->x; // current position x
	    prey = p->ref[index]->y; // current position y
	    s = e->sections[sindex]; // new section
	    prex = prex < 0 ? 0 : prex;
	    prey = prey < 0 ? 0 : prey;
	    prex = prex >= ps->X ? ps->X-1 : prex;
	    prey = prey >= ps->Y ? ps->Y-1 : prey;
	    newx = xpos < 0 ? 0 : xpos;
	    newy = ypos < 0 ? 0 : ypos;
	    newx = newx >= s->X ? s->X-1 : newx;
	    newy = newy >= s->Y ? s->Y-1 : newy;
	    // remove agent from its current cell
	    //printf("previous pos = %d,%d\n",prex,prey);
	    //printf("new pos = %d,%d\n",newx,newy);
	    if (psindex != sindex || prex != newx || prey != newy){ // different section or position
		//printf("pre agents in [%d,%d] = %d : ",prex,prey,ps->cells[prex][prey].nagents);
		//for (i=0;i<ps->cells[prex][prey].nagents;i++)
		//    printf("%d,",ps->cells[prex][prey].agids[i]);
		//printf("\n");
		//fflush(stdout);
		n = ps->cells[prex][prey].nagents;
		i = 0;
		while (i < n){
		    if (ps->cells[prex][prey].agids[i] == agid){
			for(k=i;k<n-1;k++)
			    ps->cells[prex][prey].agids[k] = ps->cells[prex][prey].agids[k+1];
			ps->cells[prex][prey].nagents--;
			break;
		    }else
			i++;
		}
		//printf("agents in [%d,%d] = %d : ",prex,prey,ps->cells[prex][prey].nagents);
		//for (i=0;i<ps->cells[prex][prey].nagents;i++)
		//    printf("%d,",ps->cells[prex][prey].agids[i]);
		//printf("\n");
		//fflush(stdout);
		//
		//printf("pre agents in [%d,%d] = %d : ",newx,newy,s->cells[newx][newy].nagents);
		//for (i=0;i<s->cells[newx][newy].nagents;i++)
		//    printf("%d,",s->cells[newx][newy].agids[i]);
		//printf("\n");
		//fflush(stdout);
		// include agent ID in the new cell
		s->cells[newx][newy].nagents++;
		//	printf("realloc? nagents=%d, maxagents = %d\n",s->cells[newx][newy].nagents,s->cells[newx][newy].maxnagents);
		if (s->cells[newx][newy].nagents >= s->cells[newx][newy].maxnagents){
		    s->cells[newx][newy].maxnagents*=2;
		    s->cells[newx][newy].agids = (int*)realloc(s->cells[newx][newy].agids,s->cells[newx][newy].maxnagents*sizeof(int));
		}
		s->cells[newx][newy].agids[s->cells[newx][newy].nagents-1] = agid;
		//printf("agents in [%d,%d] = ",newx,newy);
		//for (i=0;i<s->cells[newx][newy].nagents;i++)
		//    printf("%d,",s->cells[newx][newy].agids[i]);
		//printf("\n");
		//fflush(stdout);
	    }
	    // update agent attributes
	    //printf("update attr[%d,%d] and section %d\n",newx,newy,section);
	    p->ref[index]->x = newx;
	    p->ref[index]->y = newy;
	    p->ref[index]->section = section;
	    //	    printf("update %d attr, existing %d\n",nattr,p->ref[index]->nattr);
	    if (p->ref[index]->nattr != nattr){
		p->ref[index]->nattr = nattr;
		p->ref[index]->attr = (double*)realloc(p->ref[index]->attr,nattr*sizeof(double));
	    }
	    for(int i =0;i<nattr;i++)
		p->ref[index]->attr[i] = attr[i];
	    return 1;
	}	
    }
    return 0;
}


int AGremove(void *environment, int agid){
    ENV*e=(ENV*)environment;
    SEC *s;
    POP*p = e->population;
    int index = agid - 1;
    int sindex;
    AGENT *ag;
    int prex,prey;
    int n,i,k;
    freeID *aux;

        printf("REMOVE_AG (env %p) %d\n",environment,agid);
//    AGENT *pa;
//    printf("population size %d, refmax %d \n",p->popsize,p->refmax);
//    pa=p->l;
//    for (i=0;i<p->popsize;i++){
//	printf("%d,",pa->id);
//	pa=pa->next;
//    }
//    printf("\n");
    if (p != NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){ // agent present in the population
	    sindex = p->ref[index]->section-1;
	    s = e->sections[sindex];
	    ag = p->ref[index];
	    if (ag->prev != NULL)
		ag->prev->next = ag->next;
	    else
		p->l = ag->next;
	    if (ag->next != NULL)
		ag->next->prev = ag->prev;
	    ag->next = NULL;
	    ag->prev = NULL;
	    p->popsize--;
	    prex = p->ref[index]->x;
	    prey = p->ref[index]->y;
	    n = s->cells[prex][prey].nagents;
	    i = 0;
	    k = 0;
	    while (i < n){
		if (s->cells[prex][prey].agids[i] == agid){
		    for(k=i;k<n-1;k++)
			s->cells[prex][prey].agids[k] = s->cells[prex][prey].agids[k+1];
		    s->cells[prex][prey].nagents--;
		    break;
		}else
		    i++;
	    }
	    p->ref[index] = NULL;
	    IDput(e->freeids,ag->id);
	    free(ag);
	    //printf("population size %d, refmax %d \n",p->popsize,p->refmax);
	    //pa=p->l;
	    //for (i=0;i<p->popsize;i++){
	    //	printf("%d,",pa->id);
	    //	pa=pa->next;
	    //}
	    //printf("\n");
	    //printf(" removed ...\n");
	    return 1;
	}
    }
    //    printf(" removed ...\n");
    return 0;
}

int AGpresent(void *environment, int agid){
    ENV *e = (ENV*)environment;
    POP *p = e->population;
    int index= agid -1;
    
    //printf("AG present id=%d \n",agid);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid)
	    return 1;
    }
    return 0;
}

int AGreinit(void *environment, int agid, int flag){
    ENV *e = (ENV*)environment;
    POP *p = e->population;
    int index= agid -1;
    
    //  printf("AG reinit id=%d \n",agid);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    if (flag < 0)
		return p->ref[index]->reinit;
	    else
		p->ref[index]->reinit = flag;
	}
    }
    return -1;
}



double AGreadAttr(void *environment, int agid, int attrnum){
    ENV *e = (ENV*)environment;
    POP *p = e->population;
    int index= agid -1;
    int nattr = attrnum -1;
    
    printf("readATTR id=%d, num = %d\n",agid,attrnum);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    return p->ref[index]->attr[nattr]; 
	}
    }
    return 0;
}

double AGreadX(void *environment, int agid){
    ENV *e = (ENV*)environment;
    POP *p = e->population;
    int index= agid -1;
    
    printf("readX id=%d \n",agid);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    return p->ref[index]->x; 
	}
    }
    return 0;
}

double AGreadY(void *environment, int agid){
    ENV *e = (ENV*)environment;
    POP *p = e->population;
    int index= agid -1;
    
    printf("readY id=%d \n",agid);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    return p->ref[index]->y; 
	}
    }
    return 0;
}


int AGsetAttr(void *environment, int agid, int attrnum, double newvalue){
    ENV *e = (ENV*)environment;
    POP *p = e->population;
    int index = agid -1;
    int nattr = attrnum -1;
    
        printf("setATTR id=%d, num = %d, newvalue = %lf \n",agid,attrnum,newvalue);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    p->ref[index]->attr[nattr] = newvalue;
	    return 1;
	}
    }
    return 0;
}

int AGXnext(void *environment, int agid, int ydist){
    // returns the id of the next agent in the positive direction to agid in the X axis, but at a distance ydist in the Y axis
    ENV *e=(ENV*) environment;
    SEC *s;
    POP *p = e->population;
    double dist;
    double mindist;
    int gap;
    int *aglist;
    int auxag;
    int nextag;
    int cellx,celly;
    int checkx,checky;
    int n,i,j,k;
    int search;
    int index = agid-1;
    int sindex;


    printf("AGXnext env %p, agid %d, ydist %d\n",environment, agid, ydist);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    // current 
	    cellx = p->ref[index]->x; 
	    celly = p->ref[index]->y;
	    sindex = p->ref[index]->section-1;
	    s = e->sections[sindex];
	    printf("agent %d at [%d,%d] sindex = %d \n",agid,cellx,celly,sindex);
	    // distance from current pos to check
	    gap=0;
	    search = 1;
	    mindist = DBL_MAX;
	    nextag = 0;
	    printf("search\n");
	    while(search){
		n = 0;
		checkx = fmin(s->X-1,cellx+gap);
		checky = celly+ydist;
		n = s->cells[checkx][checky].nagents;
		if (n > 0) 
		    printf(" 0 - agents in [%d,%d] = %d\n",checkx,checky,s->cells[checkx][checky].nagents);
		aglist = (int*)malloc(n*sizeof(int));
		k = 0;
		for(j=0;j<n;j++)
		    aglist[k++] = s->cells[checkx][checky].agids[j];
		// check agents
		//printf("find NEXT of %d (k-1=%d,gap=%d)\n",agid,k-1,gap);
		for(i=0;i<k;i++){
		    auxag = aglist[i];
		    //printf("check %d \n",auxag);
		    if (auxag != agid){ // only check other agents
			dist = sqrt(pow(fabs(p->ref[auxag-1]->y - p->ref[index]->y),2.0)+pow(fabs(p->ref[auxag-1]->x - p->ref[index]->x),2.0));
			//printf(" x = %f, y = %f\n",p->ref[index]->x,p->ref[index]->y);
			//printf(" auxx = %f, auxy = %f\n",p->ref[auxag-1]->x,p->ref[auxag-1]->y);
			//printf("pos %d = [%d,%d], pos to check %d = [%d,%d], dist = %lf (mindist= %lf)\n",agid, cellx,celly, auxag, (int)p->ref[auxag-1]->x,(int)p->ref[auxag-1]->y,dist,mindist);
			if (dist > 0 && dist < mindist){ // only positive direction
			    //	    printf("nextag = %d\n",auxag);
			    mindist = dist;
			    nextag = auxag;
			}	
		    }
		}
		if (nextag){
		    printf("nextag = %d\n",nextag);
		    return nextag;
		}
		gap++;
		if (cellx+gap > s->X-1) // out of dimension, stop search
		    search = 0;
	    }
	    printf("end\n");
	}
    }
    return 0;
}

int AGXprev(void *environment, int agid, int ydist){
    // returns the id of the next agent in the positive direction to agid in the X axis, but at a distance ydist in the Y axis
    ENV *e=(ENV*) environment;
    SEC *s;
    POP *p = e->population;
    double dist;
    double mindist;
    int gap;
    int *aglist;
    int auxag;
    int nextag;
    int cellx,celly;
    int checkx,checky;
    int n,i,j,k;
    int search;
    int index = agid-1;
    int sindex;


    printf("AGXprev env %p, agid %d, ydist %d\n",environment, agid, ydist);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    // current 
	    cellx = p->ref[index]->x; 
	    celly = p->ref[index]->y;
	    sindex = p->ref[index]->section-1;
	    s = e->sections[sindex];
	    printf("agent %d at [%d,%d] sindex = %d \n",agid,cellx,celly,sindex);
	    // distance from current pos to check
	    gap=0;
	    search = 1;
	    mindist = DBL_MAX;
	    nextag = 0;
	    printf("search\n");
	    while(search){
		n = 0;
		checkx = fmax(0,cellx-gap);
		checky = celly+ydist;
		n = s->cells[checkx][checky].nagents;
		if (n > 0) 
		    printf(" 0 - agents in [%d,%d] = %d\n",checkx,checky,s->cells[checkx][checky].nagents);
		aglist = (int*)malloc(n*sizeof(int));
		k = 0;
		for(j=0;j<n;j++)
		    aglist[k++] = s->cells[checkx][checky].agids[j];
		// check agents
		//printf("find NEXT of %d (k-1=%d,gap=%d)\n",agid,k-1,gap);
		for(i=0;i<k;i++){
		    auxag = aglist[i];
		    //printf("check %d \n",auxag);
		    if (auxag != agid){ // only check other agents
			dist = sqrt(pow(fabs(p->ref[auxag-1]->y - p->ref[index]->y),2.0)+pow(fabs(p->ref[auxag-1]->x - p->ref[index]->x),2.0));
			//printf(" x = %f, y = %f\n",p->ref[index]->x,p->ref[index]->y);
			//printf(" auxx = %f, auxy = %f\n",p->ref[auxag-1]->x,p->ref[auxag-1]->y);
			//printf("pos %d = [%d,%d], pos to check %d = [%d,%d], dist = %lf (mindist= %lf)\n",agid, cellx,celly, auxag, (int)p->ref[auxag-1]->x,(int)p->ref[auxag-1]->y,dist,mindist);
			if (dist > 0 && dist < mindist){
			    //	    printf("nextag = %d\n",auxag);
			    mindist = dist;
			    nextag = auxag;
			}	
		    }
		}
		if (nextag){
		    printf("nextag = %d\n",nextag);
		    return nextag;
		}
		gap++;
		if (cellx+gap > s->X-1) // out of dimension, stop search
		    search = 0;
	    }
	    printf("end\n");
	}
    }
    return 0;
}


int AGnext(void *environment, int agid, int axis){
    // returns the id of the next agent in the positive direction to agid in the axis (0=x,1=y,2=both)
    ENV *e=(ENV*) environment;
    SEC *s;
    POP *p = e->population;
    double dist;
    double mindist;
    int gap;
    int *aglist;
    int auxag;
    int nextag;
    int cellx,celly;
    int checkx,checky;
    int n,i,j,k;
    int search;
    int index = agid-1;
    int sindex;


    printf("AGnext env %p, agid %d, axis %d\n",environment, agid, axis);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    // current 
	    cellx = p->ref[index]->x; 
	    celly = p->ref[index]->y;
	    sindex = p->ref[index]->section-1;
	    s = e->sections[sindex];
	    printf("agent %d at [%d,%d] (axis= %d) sindex = %d \n",agid,cellx,celly,axis,sindex);
	    // distance from current pos to check
	    gap=0;
	    search = 1;
	    mindist = DBL_MAX;
	    nextag = 0;
	    printf("search\n");
	    while(search){
		n = 0;
		if (axis == 0){
		    checkx = fmin(s->X-1,cellx+gap);
		    checky = celly;
		    n = s->cells[checkx][checky].nagents;
		    if (n > 0) 
			printf(" 0 - agents in [%d,%d] = %d\n",checkx,checky,s->cells[checkx][checky].nagents);
		    aglist = (int*)malloc(n*sizeof(int));
		    k = 0;
		    for(j=0;j<n;j++)
			aglist[k++] = s->cells[checkx][checky].agids[j];
		}else if (axis == 1){
		    checkx = cellx;
		    checky = fmin(s->Y-1,celly+gap);
		    n = s->cells[checkx][checky].nagents;
		    //printf(" 1 - agents in [%d,%d] = %d\n",checkx,checky,s->cells[checkx][checky].nagents);
		    aglist = (int*)malloc(n*sizeof(int));
		    k = 0;
		    for(j=0;j<n;j++)
			aglist[k++] = s->cells[checkx][checky].agids[j];
		}else{
		    checkx = fmin(s->X-1,cellx+gap);
		    checky = fmin(s->Y-1,celly+gap);
		    n = 0;
		    for(i=cellx;i<=checkx;i++){
			n += s->cells[i][checky].nagents;
			//printf(" 2.1 agents in [%d,%d] = %d\n",i,checky,s->cells[i][checky].nagents);
		    }
		    for(i=celly;i<checky;i++){
			n += s->cells[checkx][i].nagents;
			//printf(" 2.2 agents in [%d,%d] = %d\n",checkx,i,s->cells[checkx][i].nagents);
		    }
		    // find agents to check
		    aglist = (int*)malloc(n*sizeof(int));
		    k = 0;
		    //printf("number of agents at gap = %d = %d\n",gap,n);
		    for(i=cellx;i<=checkx;i++){
			n = s->cells[i][checky].nagents;
			for(j=0;j<n;j++)
			    aglist[k++] = s->cells[i][checky].agids[j];
		    }
		    for(i=celly;i<checky;i++){
			n = s->cells[checkx][i].nagents;
			for(j=0;j<n;j++)
			    aglist[k++] = s->cells[checkx][i].agids[j];
		    }
		    
		    
		}
		// check agents
		//printf("find NEXT of %d (k-1=%d,gap=%d)\n",agid,k-1,gap);
		for(i=0;i<k;i++){
		    auxag = aglist[i];
		    //printf("check %d \n",auxag);
		    if (auxag != agid){ // only check other agents
			dist = sqrt(pow(fabs(p->ref[auxag-1]->y - p->ref[index]->y),2.0)+pow(fabs(p->ref[auxag-1]->x - p->ref[index]->x),2.0));
			//printf(" x = %f, y = %f\n",p->ref[index]->x,p->ref[index]->y);
			//printf(" auxx = %f, auxy = %f\n",p->ref[auxag-1]->x,p->ref[auxag-1]->y);
			//printf("pos %d = [%d,%d], pos to check %d = [%d,%d], dist = %lf (mindist= %lf)\n",agid, cellx,celly, auxag, (int)p->ref[auxag-1]->x,(int)p->ref[auxag-1]->y,dist,mindist);
			if (dist > 0 && dist < mindist){ // only positive direction
			    //	    printf("nextag = %d\n",auxag);
			    mindist = dist;
			    nextag = auxag;
			}	
		    }
		}
		if (nextag){
		    printf("nextag = %d\n",nextag);
		    return nextag;
		}
		gap++;
		if ((axis == 0 && cellx+gap > s->X-1) ||
		    (axis == 1 && celly+gap > s->Y-1) ||
		    (axis == 2 && cellx+gap > s->X-1 && celly+gap > s->Y-1)) // out of dimension, stop search
		    search = 0;
	    }
	    printf("end\n");
	}
    }
    return 0;
}




int AGclosest(void *environment,int agid, int axis){
    // resturns the closest agent in either positive/negative directon to agid in the axis (0=x,1=y,2=both)
    ENV *e=(ENV*) environment;
    SEC *s;
    POP *p = e->population;
    double dist;
    double mindist;
    int gap;
    int *aglist;
    int auxag;
    int nextag;
    int cellx,celly;
    int fromx,tox, fromy,toy;
    int n,i,j,k,l;
    int search;
    int index = agid-1;
    int sindex;
    
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    // current cell
	    cellx = p->ref[index]->x; 
	    celly = p->ref[index]->y;
	    sindex = p->ref[index]->section-1;
	    s = e->sections[sindex];
	    // distance from current to check
	    gap=0;
	    search = 1;
	    mindist = DBL_MAX;
	    nextag = 0;
	    while(search){
		n = 0;
		if (axis == 0){ // only X
		    fromx = fmax(0,cellx-gap);
		    tox = fmin(s->X-1,cellx+gap);
		    fromy = celly;
		    toy = celly;
		    n = s->cells[fromx][fromy].nagents;
		    n += s->cells[tox][toy].nagents;
		    //printf(" 0 - agents in [%d,%d] = %d\n",checkx,checky,s->cells[checkx][checky].nagents);
		    aglist = (int*)malloc(n*sizeof(int));
		    k = 0;
		    n = s->cells[fromx][fromy].nagents;
		    for(j=0;j<n;j++)
			aglist[k++] = s->cells[fromx][fromy].agids[j];
		    n = s->cells[tox][toy].nagents;
		    for(j=0;j<n;j++)
			aglist[k++] = s->cells[tox][toy].agids[j];
		}else if (axis == 1){ // only Y
		    fromy = fmax(0,celly-gap);
		    toy = fmin(s->Y-1,celly+gap);
		    fromx = cellx;
		    tox = cellx;
		    n = s->cells[fromx][fromy].nagents;
		    n += s->cells[tox][toy].nagents;
		    //printf(" 0 - agents in [%d,%d] = %d\n",checkx,checky,s->cells[checkx][checky].nagents);
		    aglist = (int*)malloc(n*sizeof(int));
		    k = 0;
		    n = s->cells[fromx][fromy].nagents;
		    for(j=0;j<n;j++)
			aglist[k++] = s->cells[fromx][fromy].agids[j];
		    n = s->cells[tox][toy].nagents;
		    for(j=0;j<n;j++)
			aglist[k++] = s->cells[tox][toy].agids[j];
		}else{ // X and Y
		    fromx = fmax(0,cellx-gap);
		    tox = fmin(s->X-1,cellx+gap);
		    fromy = fmax(0,celly-gap);
		    toy = fmin(s->Y-1,celly+gap);
		    for(i=fromx;i<=tox;i++)
			n = s->cells[i][fromy].nagents;
		    for(i=fromx;i<=tox;i++)
			n += s->cells[i][toy].nagents;
		    for(i=fromy+1;i<toy;i++)
			n += s->cells[fromx][i].nagents;
		    for(i=fromy+1;i<toy;i++)
			n += s->cells[tox][i].nagents;
		    aglist = (int*)malloc(n*sizeof(int));
		    k = 0;
		    for(i=fromx;i<=tox;i++){
			n = s->cells[i][fromy].nagents;
			for(l=0;l<n;l++)
			    aglist[k++] = s->cells[i][fromy].agids[l];
		    }
		    for(i=fromx;i<=tox;i++){
			n = s->cells[i][toy].nagents;
			for(l=0;l<n;l++)
			    aglist[k++] = s->cells[i][toy].agids[l];
		    }
		    for(i=fromy+1;i<toy;i++){
			n = s->cells[fromx][i].nagents;
			for(l=0;l<n;l++)
			    aglist[k++] = s->cells[fromx][i].agids[l];
		    }
		    for(i=fromy+1;i<toy;i++){
			n = s->cells[tox][i].nagents;
			for(l=0;l<n;l++)
			    aglist[k++] = s->cells[tox][i].agids[l];
		    }
		}
		for(i=0;i<k;i++){
		    auxag = aglist[i];
		    if (auxag != agid){ // only check other agents
			dist = sqrt(pow(fabs(p->ref[auxag-1]->y - p->ref[index]->y),2.0)+pow(fabs(p->ref[auxag-1]->x - p->ref[index]->x),2.0));
			if (dist > 0 && dist < mindist){ // only positive direction
			    mindist = dist;
			    nextag = auxag;
			}	
		    }
		}
		if (nextag)
		    return nextag;
		gap++;
		if ((axis == 0 && cellx+gap > s->X-1) ||
		    (axis == 1 && celly+gap > s->Y-1) ||
		    (axis == 2 && cellx+gap > s->X-1 && celly+gap > s->Y-1)) // out of dimension, stop search
		    search = 0;
	    }
	}
    }
    return 0;
}

int AGneighbors(void *environment,int agid, int radius, int *neighbors){
    // returns the IDs of the neighbors of agid in a given distance in the axis (0=x,1=y,2=both);
    ENV *e=(ENV*) environment;
    SEC *s;
    POP *p = e->population;
    double dist;
    double mindist;
    int *aglist;
    int auxag;
    int nextag;
    int cellx,celly;
    int fromx,tox, fromy,toy;
    int n,i,j,k=0,l;
    int search;
    int index = agid-1;
    int sindex;
    
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    // current cell
	    cellx = p->ref[index]->x; 
	    celly = p->ref[index]->y;
	    sindex = p->ref[index]->section-1;
	    s = e->sections[sindex];
	    fromx = fmax(0,cellx-radius);
	    tox = fmin(s->X-1,cellx+radius);
	    fromy = fmax(0,celly-radius);
	    toy = fmin(s->Y-1,celly+radius);
	    n=0;
	    for(i=fromx;i<=tox;i++){
		for(j=fromy;i<=toy;j++)
		    n += s->cells[i][j].nagents;
	    }
	    aglist = (int*)malloc(n*sizeof(int));
	    k = 0;
	    for(i=fromx;i<=tox;i++){
		for(j=fromy;i<=toy;j++){
		    n = s->cells[i][j].nagents;
		    for(l=0;l<n;l++)
			aglist[k++] = s->cells[i][j].agids[l];
		}
	    }
	    neighbors = (int*)malloc(k-1*sizeof(int));
	    for(i=0;i<k-1;i++)
		neighbors[i] = aglist[i];
	}
    }
    return k-1;
}


#endif
