/***************************************************************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the LGPL License
* Copyright © 2013, Victorino Sanz.
* 
****************************************************************************************************/

#ifndef FURNACE
#define FURNACE

#include <CellularAutomataLib.c>

/****************************************/
/* FURNACE CELLULAR AUTOMATA       */
/****************************************/

//**************************************
// Cell state
typedef struct FURState{
    double t;
}FURState;



//**************************************
// CA WRAPPER FUNCTIONS

// Function that allocates memory and sets the default state for the cells
void * FURDefaultState(){
    FURState *s;
    s = (FURState *)malloc(sizeof(FURState));
    
    s->t = 0;
    return (void *)s;
}

// function to set default values in the cellular space
int FURDefault(void *space){
    void* (*defaultState)() = &FURDefaultState;
    CS2D_InitDefault(space,defaultState);
    return 1;
}

// Function that describes the state of initialized cells
void *FURInitialState(){
   FURState *s;
    s = (FURState *)malloc(sizeof(FURState));
    
    s->t = 1;
    return (void *)s;
}

// initializing function for the selected cell
void FURInitial(void* space, int row, int col){
    void *(*initialState)() = &FURInitialState;
    CS2D_Init(space, row, col, initialState);
    return;
}

//***************************************************
// RULE FUNCTION
void *furnace(int*change, void* cellstate, int row, int col, void** neighbors, int n_neighbors, void** inputs, int n_inputs, int *inputs_rcv){
    double sum;
    int i;
    FURState *out;
    FURState *cs;
    FURState **ne;
    FURState **in;
    
    cs = (FURState *)cellstate;
    ne = (FURState **)neighbors;
    in = (FURState **)inputs;
    out = (FURState *)malloc(sizeof(FURState));
    
    sum = 0;
    for(i=0;i<n_neighbors;i++){
        if(neighbors[i] != NULL){
            sum += ne[i]->t;
        }
    }

     
    out->t = cs->t;
    if (cs->t > sum/n_neighbors)
        out->t = cs->t;
    else 
        out->t = (sum/n_neighbors);

 
    if (in != NULL)
	for (i=0;i<n_inputs;i++)
	    if (inputs_rcv[i] > 0)
		out->t = in[i]->t;
    
    *change = 1;
    if (out->t == cs->t)
	*change = 0;
    // ModelicaFormatMessage("S = %f, SUM = %f, NEW S = %f , CHANGE = %d \n",cs->t,sum,out->t,*change);
    
    return (void *)out;
}

// Function that returns the value to display in the graphical animation
double FURDisplay(void *cell){
    FURState *s;

    s = (FURState *)cell;
    if (s == NULL)
	return 0;
    else
	return (double)s->t;
}

int FURStep(void* space){
    void* (*rule)(int*, void*,int,int,void**,int,void**,int,int*) = &furnace;
    double (*display)(void*) = &FURDisplay;
    
    return CS2D_Step(space,rule,display);
    //return 1;
}

/*********************************************/

void FURExtInput(void* space,int row, int col, double value,int input_id){
    FURState* inp;

    inp = (FURState *)malloc(sizeof(FURState));
    inp->t = value;
    CS2D_SetInput(space,row,col,(void*)inp,input_id);

    return;
}

double FUROutput(void* space, int row, int col){
    FURState* s;

    s = (FURState *)CS2D_GetState(space,row,col);
    return s->t;
}

void FURSetInput(void* Fspace, int Frow, int Fcol, void* Tspace, int Trow, int Tcol, int input_id){
    FURState* s;
    FURState* inp;
    
    inp = (FURState *)malloc(sizeof(FURState));
    s = (FURState *)CS2D_GetState(Fspace,Frow,Fcol);
    *inp = *s;
    CS2D_SetInput(Tspace,Trow,Tcol,(void*)inp,input_id);
}

#endif
