/***************************************************************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the LGPL License
* Copyright © 2013, Victorino Sanz.
* 
****************************************************************************************************/

#ifndef GOL
#define GOL

#include <CellularAutomataLib.c>

/****************************************/
/* GAME OF LIFE CELLULAR AUTOMATA       */
/****************************************/

//**************************************
// Cell state
typedef struct GOLState{
    int v;
}GOLState;

//**************************************
// CA WRAPPER FUNCTIONS

// Function that allocates memory and sets the default state for the cells
void * GOLDefaultState(){
    GOLState *s;
    s = (GOLState *)malloc(sizeof(GOLState));
    
    s->v = 0;
    return (void *)s;
}

// function to set default values in the cellular space
int GOLDefault(void *state){
    void* (*defaultState)() = &GOLDefaultState;
    CS2D_InitDefault(state,defaultState);
    return 1; 
}

// Function that describes the state of initialized cells
void* GOLInitialState(){
    GOLState *s;
    s = (GOLState*)malloc(sizeof(GOLState));
    s->v = 1;   
    return (void*)s;
}


// initializing function for the selected cell
void GOLInitial(void* space, int row, int col){
    void * (*initialState)() = &GOLInitialState;
    CS2D_Init(space, row, col, initialState);
    return;
}

// Function that describes the state of initialized cells
void* GOLRandomInitialState(){
    GOLState *s;
    time_t t;
    
    s = (GOLState *)malloc(sizeof(GOLState));;
    /* Intializes random number generator */
  
    //    srand((unsigned) time(&t));
    s->v = random() % 2;
    return (void*)s;
}


// initializing function for the selected cell
void GOLRandomInitial(void* space, int row, int col){
    int i,j;

    void*  (*initialState)() = &GOLRandomInitialState;
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++){
	    CS2D_Init(space, i, j, initialState);
	}
    return;
}

//***************************************************
// RULE FUNCTION
void* gol(int* change, void* cellstate, int row, int col, void** neighbors, int n_neighbors, void** inputs, int n_inputs, int *inputs_rcv){
    int sum,i;
    GOLState *out;
    GOLState *cs;
    GOLState **ne;
    GOLState **in;
    
    cs = (GOLState *)cellstate;
    ne = (GOLState **)neighbors;
    in = (GOLState **)inputs;
    out = (GOLState *)malloc(sizeof(GOLState));
    
    sum = 0;
    for(i=0;i<n_neighbors;i++){
        if(neighbors[i] != NULL){
            sum += ne[i]->v;
        }
    }
    
    out->v = cs->v;
    if (!cs->v && sum == 3)
        out->v = 1;
    else if (cs->v && (sum < 2 || sum > 3 ) )
        out->v = 0;
    
    *change = 1;
    if(cs->v == out->v)
	*change = 0;
    
    return (void *)out;
}

// Function that returns the value to display in the graphical animation
double GOLDisplay(void *cell){
    GOLState *s;

    if(cell == NULL)
	return -1;
    else{
	s = (GOLState *)cell;
	return (double)s->v;
    }
}

int GOLStep(void* space){
    void* (*rule)(int*,void*,int, int, void**,int,void**,int,int*) = &gol;
    double (*display)(void*) = &GOLDisplay;
    
    return CS2D_Step(space,rule,display);
    //return 1;
}

#endif
