#ifndef EXTPDEVS
#define EXTPDEVS

#define INITNDEST 5
#define INITMAXSIZE 10

//#include <msglib.c>


typedef struct{
    int max_size; // maximum number of messages (can be extended, only used in reallocs)
    int size; // number of messages
    double ** msgs; // messages
    int *msglen; // length of each message in the buffer
    int ndest; // number of destinations
    struct port ** dest; // destinations
    void *M; // model that includes this port
}port;

/*********************************************************************************/

port * initPort(void *M){
    port* p = (port*)malloc(sizeof(port));
    p->ndest = 0;
    p->dest = (port**)malloc(INITNDEST*sizeof(port*));
    p->max_size = INITMAXSIZE;
    p->msgs = (double **)malloc(b->max_size*sizeof(double*));
    p->msglen = (int *)malloc(b->max_size*sizeof(int));
    p->size = 0;
    p->M = M;
    return (port*) p;
}

/*********************************************************************************/
void closePort(port* o){
    //  free(b->msgs);
    //free(b->dest);
    free(o);
}

/*********************************************************************************/


int size(port* o){
    return o->size;
}


/*********************************************************************************/

void * model(port* o){
    return o->M;
}


/*********************************************************************************/

void couple(port* o, port* d){
    o->ndest++;
    if (o->ndest > INITNDEST){ // initially allocated memory for five destinations in initBuffer
	o->dest = (port**)realloc(o->dest,o->ndest*sizeof(port*));	
    }
    o->dest[o->ndest-1] = d;
}

/*********************************************************************************/


// put a new msg in the buffer o at position mindex
void put(double *msg, size_t len, port* o, int mindex){
    port* d;
    int i,j,k;
    int rindex;
    int index = mindex -1; //modelica first index == 1
    int rmax = 10;
    port** r = (port**)malloc(rmax*sizeof(port*)); // list of destination buffers that also have destinations (for coupled models).

    //printf("BPUT mindex:%d\n",mindex);
    // store msg locally	
    if (o->size == o->max_size){
	o->max_size = o->max_size*2;
	o->msgs = (double**)realloc(o->msgs,o->max_size*sizeof(double*));
	o->msglen = (int*)realloc(o->msglen,o->max_size*sizeof(int));
    }	
    o->size++;
    if (index < 0 || index == o->size-1){ // msg to the end of the buffer
	o->msglen[o->size-1] = len;
	o->msgs[o->size-1] = (double*)malloc(len*sizeof(double));
	for(i=0;i<len;i++)
	    o->msgs[o->size-1][i] = msg[i];
    }else{ // msg to a middle position
	for(i=o->size-1;i>index;i--){
	    o->msgs[i] = o->msgs[i-1];
	}
	o->msglen[index] = len;
	o->msgs[index] = (double*)malloc(len*sizeof(double));
	for(k=0;k<len;k++)
	    o->msgs[index][k] = msg[k];
    }
    free(r);
}




/*************************************************************************************************/


// get the position mpos of indexed msg from the buffer, removing the whole msg
double get(port* o, int mpos, int mindex){
    double * out;
    int i;
    int index = mindex -1;
    int pos = mpos -1;
    
    if (o->size > index){
	out = o->msgs[index];
	for(i=index;i<o->size;i++){
	    o->msgs[i] = o->msgs[i+1];
	}
	o->size--;
	return out[pos];	
    }
}


/******************************************************************************************************/

// read the position mpos of the indexed msg from the buffer
double read(port* o, int mpos, int mindex){
    double * out;
    int i;
    int index = mindex -1;
    int pos = mpos -1;

    //printf("BREAD mpos:%d, mindex:%d\n",mpos,mindex);
    if (o->size >= index){
	out = o->msgs[index];
	return out[pos];
    }       
}

/******************************************************************************************************/

// write a new value for mpos of the indexed msg from the buffer
double write(port* o, double newval, int mpos, int mindex){
    double * out;
    int i;
    int index = mindex -1;
    int pos = mpos -1;
    double preval;

    //printf("BREAD mpos:%d, mindex:%d\n",mpos,mindex);
    if (o->size >= index && o->msglen >= pos){
	out = o->msgs[index];
	preval = out[pos];
	out[pos] = newval;
    }else
	preval = -1;
    return preval;
    
}



/*********************************************************************************/

// send msgs from its origin port to all the destinations
int send(port* o,void **Mdest){
    port* d;
    int i,j,k;
    int rindex;
    // int index = mindex -1; //modelica first index == 1
    int rmax = 10;
    port** r = (port**)malloc(rmax*sizeof(port*)); // list of destination buffers that also have destinations (for coupled models).
    double **msgs;
    int *msgsize;
    int psize;
    int ndest;
    int maxdest = INITNDEST;

    psize = size(o);
    msgs = (double **)malloc(psize*sizeof(double*));
    msgsize = (int *)malloc(psize*sizeof(int));
    for (i=0;i<psize;i++){
	msgsize[i] = o->msglen[i];
	msgs[i] = (double*)malloc(msgsize[i]*sizeof(double));
	for(j=0;j<msgsize[i]-1;j++){
	    msgs[i][j] = read(o,j+1,1); // read positions of the array
	}
	msgs[i][j] = get(o,j+1,1); // get last position and remove msg from buffer
    }

    ndest = 0;
    Mdest = (void**)malloc(maxdest*sizeof(void*));
    if (o->ndest > 0){ // send msg to destinations
	rindex = 0;
	j = 0;
	r[0] = o;
	while(j <= rindex){ // for each buffer in the routing list
	    for (i = 0;i<r[j]->ndest;i++){ // for each destination in the current buffer
		d = (port*)r[j]->dest[i];
		if (d->ndest > 0){// include buffer in routing
		    rindex++;
		    if (rindex == rmax){ // realloc r if index reaches max
			rmax = rmax*2;
			r = (port**)realloc(r,rmax*sizeof(port*));
		    }
		    r[rindex] = d;
		}else{ // copy message in real destination (not routing) and store destination model in Mdest
		    for (k = 0;k<psize;k++)
			bputA(msgs[k],msgsize[k],d,0);
		    Mdest[ndest++] = d->M;
		    if (ndest > maxdest){
			maxdest *= 2;
			Mdest = (void**)realloc(Mdest,maxdest*sizeof(void*));
		    }
		}
	    }
	    j++;
	}
    }
    free(r);
    return ndest;
}






#endif










