/**********************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the Modelica License 2
* Copyright © 2013, Victorino Sanz.
* 
***********************************************************/

#ifndef POPULATION
#define POPULATION


typedef struct AGENT{
    int id;
    double order;
    int x,y;
    int nattr;
    double *attr;
    struct AGENT *next;
    struct AGENT *prev;
} AGENT;

typedef struct POP{
    int popsize; // population size
    AGENT* pop; // population of agents
    int refmax; // maximum agent ID in the population
    int initags; // initialization of agent references
    AGENT** ref; // list of direct references to agents
} POP;


void *createPOP(int initags){
    POP *p;
    int i;

    //printf("CREATE_POP\n");
    p = (POP*)malloc(sizeof(POP));
    p->popsize = 0;
    p->pop = NULL;
    p->refmax = initags;
    p->ref = (AGENT**)malloc(initags*sizeof(AGENT*));
    for(i=0;i<initags;i++)
	p->ref[i] = NULL;
    
    return (void *)p;
}

int deletePOP(void *population){
    POP *p;
    
    //printf("DELETE_POP\n");
    p = (POP*)population;
    if (p->pop != NULL)
	free(p->pop);
    if (p->ref != NULL)
	free(p->ref);
    p->pop = NULL;
    return 1;
}

int insertAG(void *population, int agid, double order, double *agattr, size_t nattr){
    POP *p = (POP*)population;
    AGENT *newag;
    AGENT *pa;
    int i;
    int index = agid-1;

    //   printf("INSERT_AG\n");
    if (p != NULL){
	if (p->ref[index] == NULL){ // new agent
	    newag = (AGENT*)malloc(sizeof(AGENT));
	    newag->id = agid;
	    newag->x = 1;
	    newag->y = 1;
	    newag->order = order;
	    newag->nattr = nattr;
	    newag->attr = (double*)malloc(nattr*sizeof(double));
	    for(i=0;i<nattr;i++) // assign attributes
		newag->attr[i] = agattr[i];
	    if(agid > p->refmax){ // update list of direct references to agents
		p->ref = (AGENT**)realloc(p->ref,2*agid*sizeof(AGENT*));   
		for(i=p->refmax;i<2*agid;i++)
		    p->ref[i] = NULL;
		p->refmax = 2*agid;
	    }
	    p->popsize++;
	}else{ // replace existing agent
	    newag = p->ref[index];
	    newag->prev->next = newag->next; // reorder list of agents
	    newag->next->prev = newag->prev;
	    newag->next = NULL; // extract newag from list
	    newag->prev = NULL;
	    if (nattr != newag->nattr){ // update array of attributes
		newag->nattr = nattr;
		newag->attr = (double*)realloc(newag->attr,nattr*sizeof(double));
	    }
	    for(i=0;i<nattr;i++) // assign new attribute values
		newag->attr[i] = agattr[i];
	}
	p->ref[index] = newag;
	pa = p->pop;
	if (pa == NULL || pa->order > order){ // empty population or insert first
	    newag->next = pa;
	    if (pa != NULL)
		pa->prev = newag;
	    newag->prev = NULL;
	    p->pop = newag;
	}else{ // find the insertion place
	    newag->prev = pa;
	    pa = pa->next;
	    while(pa != NULL){
	        if(pa->order > order){ // insert before pa
		    newag->next = pa;
		    newag->prev = pa->prev;
		    pa->prev->next = newag;
		    pa->prev = newag;
		    return 1;
		}else{ // check next
		    newag->prev = pa;
		    pa = pa->next;
		}
	    } // insert at the end;
	    newag->next = NULL;
	    newag->prev->next = newag;
	}
    }
    return 1;
}

int updateAG(void *population, int agid, double *attr, size_t nattr){
    POP *p = (POP*)population;
    AGENT *newag;
    AGENT *pa;
    int i;
    int index = agid-1;

    //printf("UPDATE_AG id=%d, agtype = %d, attr[7] = %lf, nattr = %lu\n",agid,agtype,attr[7],nattr);
    if (p != NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){ // agent present in the population
	    if (p->ref[index]->nattr != nattr){
		p->ref[index]->nattr = nattr;
		p->ref[index]->attr = (double*)realloc(p->ref[index]->attr,nattr*sizeof(double));
	    }
	    for(int i =0;i<nattr;i++)
		p->ref[index]->attr[i] = attr[i];
	    return 1;
	}	
    }
    return 0;
}

int removeAG(void *population, int agid){
    POP*p = (POP*)population;
    int index = agid - 1;
    AGENT *ag;

    //    printf("REMOVE_AG\n");
    if (p != NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){ // agent present in the population
	    ag = p->ref[index];
	    if (ag->prev != NULL)
		ag->prev->next = ag->next;
	    if (ag->next != NULL)
		ag->next->prev = ag->prev;
	    p->popsize--;
	    p->ref[index] = NULL;
	    free(ag);
	    return 1;
	}
    }
    return 0;
}

double readAttr(void *population, int agid, int attrnum){
    POP *p=(POP*)population;
    int index= agid -1;
    int nattr = attrnum -1;

    //printf("readATTR id=%d, num = %d\n",agid,attrnum);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    return p->ref[index]->attr[nattr]; 
	}
    }
    return 0;
}

double readNeighborAttr(void *population, int agid, int attrnum, int distance){
    POP *p=(POP*)population;
    int index= agid -1;
    int nattr = attrnum -1; 
    AGENT *ag;

    //printf("readATTR id=%d, num = %d\n",agid,attrnum);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL && p->ref[index]->id == agid){
	    ag = p->ref[index];
	    if (distance >= 0){
		for(i=0;i<distance;i++){
		    if (ag->next != NULL)
			ag = ag->next;
		    else
			return -1; // out of the population 
		}	
	    }else{
		for(i=0;i<distance;i++){
		    if (ag->prev != NULL)
			ag = ag->prev;
		    else
			return -1; // out of the population 
		}	
	    }
	    return ag->attr[nattr]; 
	}
    }
    return 0;
}

int setAttr(void *population, int agid, int attrnum, double value){
    POP *p=(POP*)population;
    int index = agid -1;
    int nattr = attrnum -1;

    //    printf("stATTR id=%d, num = %d, newvalue = %lf \n",agid,attrnum,value);
    if(p!=NULL){
	if (agid <= p->refmax && p->ref[index] != NULL){
	    p->ref[index]->attr[nattr] = value;
	    return 1;
	}
    }
    return 0;
}

int popsize(void *population){
    POP*p =(POP*)population;

    //    printf("POPSIZE \n");
    return p->popsize;
    
}

#endif
