/***************************************************************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the LGPL License
* Copyright © 2021, Victorino Sanz.
* 
****************************************************************************************************/

#ifndef SHEEP
#define SHEEP

#include <CellularAutomataLib.c>
#include <stdlib.h>
#include <time.h>

/****************************************/
/* SHEEP-WOLVES CELLULAR AUTOMATA       */
/****************************************/

//**************************************
// Cell state
typedef struct SheepState{
    double grass;
    double sheep;
}SheepState;

//**************************************
// CA WRAPPER FUNCTIONS

// Function that allocates memory and sets the default state for the cells
void * SheepDefaultState(){
    SheepState *s;
    s = (SheepState *)malloc(sizeof(SheepState));
    
    s->grass = rand()%5;
    s->sheep = 0;
    return (void *)s;
}

// function to set default values in the cellular space
int SheepDefault(void *state){
    void* (*defaultState)() = &SheepDefaultState;
    CS2D_InitDefault(state,defaultState);
    return 1; 
}
//****************************************************

// Function that describes the state of initialized cells
void* SheepRandomInitialState(){
    SheepState *s;
    time_t t;
    
    s = (SheepState *)malloc(sizeof(SheepState));
    s->grass = random() % 5;
    s->sheep = 0;
   
    return (void*)s;
}


// initializing function for the selected cell
void SheepRandomInitial(void* space, int row, int col){
    int i,j;
    time_t t;
    
    void*  (*initialState)() = &SheepRandomInitialState;
    /* Intializes random number generator */
    srand((unsigned) time(&t));
    for(i=1;i<=row;i++)
	for(j=1;j<=col;j++){
	    CS2D_Init(space, i, j, initialState);
	}
    return;
}

//***************************************************
// RULE FUNCTION
void* grassgrow(int* change, void* cellstate, int row, int col, void** neighbors, int n_neighbors, void** inputs, int n_inputs, int *inputs_rcv){
    int sum,i;
    SheepState *out;
    SheepState *cs;
    SheepState **ne;
    SheepState **in;
    
    cs = (SheepState *)cellstate;
    ne = (SheepState **)neighbors;
    in = (SheepState **)inputs;
    out = (SheepState *)malloc(sizeof(SheepState));
 
    out->grass = cs->grass+1;
    *change = 1;

    return (void *)out;
}

// Function that returns the value to display in the graphical animation
double SheepDisplay(void *cell){
    SheepState *s;

    if(cell == NULL)
	return -1;
    else{
	s = (SheepState *)cell; 
	return (double)s->sheep;
    }
}

int SheepStep(void* space){
    void* (*rule)(int*,void*,int, int, void**,int,void**,int,int*) = &grassgrow;
    double (*display)(void*) = &SheepDisplay;
    
    return CS2D_Step(space,rule,display);
    //return 1;
}

//*********************************************************

// function to convert the state of a cell into a double value
double OutputGrass(void* space, int row, int col){
    double out;
    SheepState* c;

    c = (State *)CS2D_GetState(space,row,col);
    //convert the external value to values of the state variables
    out = c->grass;
    
    return out;
}

// function to convert the state of a cell into a double value
double OutputSheep(void* space, int row, int col){
    double out;
    SheepState* c;

    c = (State *)CS2D_GetState(space,row,col);
    //convert the external value to values of the state variables
    out = c->sheep;
    
    return out;
}


//***************************************************************+

// function to set an external value as an input to a cell
void RemoveGrass(void* space,int row, int col, double value,int input_id){
    double* inp;

    inp = (double *)malloc(sizeof(double));
    //convert the external value to values of the state variables
    *inp = value;

    CS2D_SetInput(space,row,col,(void*)inp,input_id);

    return;
}

void UpdateSheep(void* space,int row, int col, double value,int input_id){
    double* inp;

    inp = (double *)malloc(sizeof(double));
    //convert the external value to values of the state variables
    *inp = value;

    CS2D_SetInput(space,row,col,(void*)inp,input_id);

    return;
}


#endif
