/***************************************************************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the LGPL License
* Copyright © 2013, Victorino Sanz.
* 
****************************************************************************************************/

#ifndef WOLFRAM
#define WOLFRAM

#include <CellularAutomataLib.c>

/****************************************/
/* WOLFRAM RULES CELLULAR AUTOMATA      */
/****************************************/

// data structure to define the state of the cell as a "int"
typedef struct RState{
    int v;
}RState;

void * RDefaultState(){
    RState *s;
    s = (RState *)malloc(sizeof(RState));
    s->v = 0; // default state to 0
    return (void *)s;
}

// REDECLARES Default IN CellSpace2D model
int RSetDefault(void* space){
    void* (*defaultState)() = &RDefaultState;
    return CS1D_SetDefault(space,defaultState);
}

void* RInitialState(){
    RState *s;
    s = (RState *)malloc(sizeof(RState));
    s->v = 1; // initial state to 1
    return (void*)s;
}

// REDECLARES Initial IN CellSpace2D model
void RInitial(void* space, int col){
    void*  (*initialState)() = &RInitialState;
    CS1D_Init(space, col, initialState);
    return;
}

// RULE FUNCTIONS

void* rule110(int* change, void* cellstate, int pos, void** neighbors, int n_neighbors,int *neighbors_present,void** inputs, int n_inputs, int *inputs_rcv){

    int n0,n1,i;
    RState *out;
    RState *cs;
    RState **ne;
    RState **in;
    
    cs = (RState *)cellstate;
    ne = (RState **)neighbors;
    in = (RState **)inputs;
    out = (RState *)malloc(sizeof(RState));
    
    if (neighbors_present[0] > -1 && ne[0]->v > 0)
	n0 = 1;
    else
	n0 = 0;
    
    if (neighbors_present[1] > -1 && ne[1]->v > 0)
	n1 = 1;
    else
	n1 = 0;
    
    
    if(n0 == 1 && cs->v == 1 && n1 == 1){
	out->v = 0;
    }else if(n0 == 1 && cs->v == 1 && n1 == 0){
	out->v = 1;
    }else if(n0 == 1 && cs->v == 0 && n1 == 1){
	out->v = 1;
    }else if(n0 == 1 && cs->v == 0 && n1 == 0){
	out->v = 0;
    }else if(n0 == 0 && cs->v == 1 && n1 == 1){
	out->v = 1;
    }else if(n0 == 0 && cs->v == 1 && n1 == 0){
	out->v = 1;
    }else if(n0 == 0 && cs->v == 0 && n1 == 1){
	out->v = 1;
    }else if(n0 == 0 && cs->v == 0 && n1 == 0){
	out->v = 0;
    }
    

    for(i=0;i<n_inputs;i++){
	//    if(n_inputs > 0){
	if ((in != NULL) && (inputs_rcv[i] > 0)){
	    out->v = in[i]->v;
	}
    }
    
    *change = 1;
    if(cs->v == out->v)
	*change = 0;
    
    return (void *)out;
}

// Function that returns the value to display in the graphical animation
double RDisplay(void *cell){
    RState *s;

    if (cell == NULL)
	return -1;
    else{
	s = (RState *)cell;
	return (double)s->v;
    }
}


int R110Step(void* space){
    void* (*rule)(int*,void*,int,void**,int,int*,void**,int,int*) = &rule110;
    double (*display)(void*) = &RDisplay;
    
    CS1D_Step(space,rule,display);
    return 1;
}

/*************************************************************************/

void* rule30(int* change, void* cellstate, int pos, void**neighbors, int n_neighbors, int *neighbors_present, void**inputs, int n_inputs, int *inputs_rcv){

    int n0,n1,i;
    RState *out;
    RState *cs;
    RState **ne;
    RState **in;
    
    cs = (RState *)cellstate;
    ne = (RState **)neighbors;
    in = (RState **)inputs;
    out =(RState *)malloc(sizeof(RState));

    if (neighbors_present[0] > -1 && ne[0]->v > 0)
	n0 = 1;
    else
	n0 = 0;
    
    if (neighbors_present[1] > -1 && ne[1]->v > 0)
	n1 = 1;
    else
	n1 = 0;
    
    if(      n0 == 1 && cs->v == 1 && n1 == 1){
	out->v = 0;
    }else if(n0 == 1 && cs->v == 1 && n1 == 0){
	out->v = 0;
    }else if(n0 == 1 && cs->v == 0 && n1 == 1){
	out->v = 0;
    }else if(n0 == 1 && cs->v == 0 && n1 == 0){
	out->v = 1;
    }else if(n0 == 0 && cs->v == 1 && n1 == 1){
	out->v = 1;
    }else if(n0 == 0 && cs->v == 1 && n1 == 0){
	out->v = 1;
    }else if(n0 == 0 && cs->v == 0 && n1 == 1){
	out->v = 1;
    }else if(n0 == 0 && cs->v == 0 && n1 == 0){
	out->v = 0;
    }
    
    for(i=0;i<n_inputs;i++){
	//    if(n_inputs > 0){
	if ((in != NULL) && (inputs_rcv[i] > 0)){
	    out->v = in[i]->v;
	}
    }
    
    *change = 1;
    if(cs->v == out->v)
	*change = 0;
    
    return (void*)out;
}

int R30Step(void* space){
    void* (*rule)(int*, void*,int, void**,int,int*,void**,int,int*) = &rule30;
    double (*display)(void*) = &RDisplay;
    
    CS1D_Step(space,rule,display);
    return 1;
}



void RSetInput(void* Fspace, int Fcol, void* Tspace, int Tcol, int input_id){
    RState* state;
    RState* input;
    
    input = (RState *)malloc(sizeof(RState));
    state = (RState *)CS1D_GetState(Fspace,Fcol);
    *input = *state;
    CS1D_SetInput(Tspace,Tcol,(void*)input,input_id);
    return;
}


#endif
